import os
import logging
import sys
from os.path import expanduser

class LogManager:

    @property
    def LOG(self):
        return self._LOG

    def __init__(self):
        self._LOG = logging.getLogger("app." + __name__)
        self._LOG.setLevel(logging.INFO)
        FILE_LOG = os.path.join(expanduser("~"), ".awsee","log")

        if not os.path.exists(FILE_LOG):
           os.makedirs(FILE_LOG)

        rootLogger = logging.getLogger()
        logFormatter = logging.Formatter("%(asctime)s [%(threadName)-12.12s] [%(levelname)-5.5s]  %(message)s")

        fileHandler = logging.FileHandler("{0}/{1}.log".format(FILE_LOG, "awsee"))
        fileHandler.setFormatter(logFormatter)
        fileHandler.setLevel(logging.INFO)
        rootLogger.addHandler(fileHandler)



import re, uuid
from tinydb import Query, where
from arnparse import arnparse
from awsee.messages import Messages
from awsee.style import Style
from awsee.emoticons import Emoticons
from awsee.utils import Utils
from awsee.tableargs import TableArgs
from awsee.prettytable import PrettyTable
from awsee.repository import CredentialsRepository, RoleRepository
from awsee.general import *

class RoleManager:

    def __init__(self):
        self.credentialsRepository = CredentialsRepository()
        self.roleRepository        = RoleRepository()

    def addRole(self):
        Messages.showMessage("ADD ROLE TO PROFILE")
        index            = 0
        sizeSeparator    = 120
        sizeLabelProfile = 18
        output           = ""
        output           += Style.GREEN + "-".ljust(sizeSeparator,"-") + "\n"
        output           += f" {Style.IBLUE}PROFILES:\n"
        profiles          = []
        for c in self.credentialsRepository.all():
            index += 1
            profiles.append(c['profile'])

            pn        = c['profile'] + " ".rjust(sizeLabelProfile -len(c['profile']), " ")
            account   = c['account']
            line      = f"  {Style.IBLUE}{index:02d} -->{Style.GREEN} {pn} {Style.IBLUE}Account..:{Style.GREEN} {account}"
            labelRole = "Roles..: "
            roles     = ""
            for idx, r in enumerate(self.roleRepository.searchByQuery(where('profile') == c['profile'])):
                if len(roles) > 1:
                    roles += "\n" + " ".ljust(len(Utils.removeCharsColors(line)) + len(labelRole) + 1," ")
                roles += f"{Style.BLUE}{idx+1:02d}{Style.GREEN}-{r['role-arn']} {Style.BLUE}({r['role-name']})"
            if len(roles) > 1:
                roles = f"{Style.IBLUE}{labelRole}{Style.GREEN}{roles}{Style.GREEN}"
            output += line + f" {roles}\n"
        output += "-".ljust(sizeSeparator,"-")
        printPy(output)
        printPy(f"{Style.GREEN} {Emoticons.pin()} Choose a profile....:{Style.BLUE}", end= ' ')
        result = input()

        if result == "":
            return None
            
        if not Utils.isNumber(result):
            Messages.showWarning(f"Invalid choice \"{Style.IBLUE}{result}{Style.IGREEN}\"")
            return None
        if int(result) > index:
            Messages.showWarning(f"Profile \"{Style.IBLUE}{result}{Style.IGREEN}\" not valid!")
            return None

        profile  = profiles[int(result)-1]
        printPy(f"{Style.GREEN} {Emoticons.pin()} Role Name...........:{Style.BLUE}",end=' ')
        roleName = input()

        spacesFound = re.search(" ",roleName)
        if spacesFound:
            Messages.showWarning(f"Invalid Role Name \"{Style.IBLUE}{roleName}{Style.IGREEN}\", spaces are not allowed")
            return None

        printPy(f"{Style.GREEN} {Emoticons.pin()} Role ARN............:{Style.BLUE}",end=' ')
        roleArn  = input()
        try:
            roleArnParse = arnparse(roleArn)
        except:
            Messages.showWarning(f"Invalid Role ARN \"{Style.IBLUE}{roleArn}{Style.IGREEN}\"")
            return None

        output  = "\n"
        output += f" {Style.IBLUE} --> {Style.GREEN}Profile............: {Style.IBLUE}{profile}{Style.RESET}\n"
        output += f" {Style.IBLUE} --> {Style.GREEN}Role Name..........: {Style.IBLUE}{roleName}{Style.RESET}\n"
        output += f" {Style.IBLUE} --> {Style.GREEN}Role ARN...........: {Style.IBLUE}{roleArn}{Style.RESET}\n"
        printPy(output)

        printPy(f"{Style.GREEN} {Emoticons.pin()} Confirm [{Style.IGREEN}y/{Style.IGREEN}n{Style.RESET}]:{Style.BLUE}",end=' ')
        confirm = input()
        if confirm.lower() == "y":
            record = {
                "id": str(uuid.uuid4()),
                "profile": profile,
                "role-name": roleName,
                "role-arn": roleArn
            }
            self.roleRepository.insert(record)
            output = f"{Style.GREEN} {Emoticons.ok()} Saved!{Style.RESET}"
            printPy(output)
    
    def removeRole(self, numberInList, roleName):
        roleToRemove = None
        if numberInList > 0:
            for idx,role in enumerate(self.roleRepository.all()):
                if (idx+1) == numberInList:
                    roleToRemove = role
                    break
        else:
            roleToRemove = self.roleRepository.searchByQuery(where('role-name') == roleName)
            roleToRemove = roleToRemove[0]
        
        if not roleToRemove:
            Messages.showWarning("Role not found!")
        else:
            output  = "\n"
            output += f" {Style.IBLUE} --> {Style.GREEN}Profile............: {Style.IBLUE}{roleToRemove['profile']}{Style.RESET}\n"
            output += f" {Style.IBLUE} --> {Style.GREEN}Role Name..........: {Style.IBLUE}{roleToRemove['role-name']}{Style.RESET}\n"
            output += f" {Style.IBLUE} --> {Style.GREEN}Role ARN...........: {Style.IBLUE}{roleToRemove['role-arn']}{Style.RESET}\n"
            printPy(output)

            printPy(f"{Style.GREEN} {Emoticons.pin()} Remove [{Style.IGREEN}y/{Style.IGREEN}n{Style.RESET}]:{Style.BLUE}",end=' ')
            confirm = input()
            if confirm.lower() == "y":
                self.roleRepository.remove(Query().id == roleToRemove['id'])
                output  = "\n"
                output += f"{Style.GREEN} {Emoticons.ok()} Removed!{Style.RESET}"
                printPy(output)
    
    def listRoles(self, filterByProfile):
        listRoles = None
        if filterByProfile:
            listRoles = self.roleRepository.searchByQuery(where('profile') == filterByProfile)
        else:
            listRoles = self.roleRepository.all()

        if len(listRoles) < 1:
            if filterByProfile:
               msg = f"There's no role added to the profile {Style.IBLUE}{filterByProfile}{Style.RESET}"
            else:
               msg = f"There's no role added"
            Messages.showWarning(msg)
            return None
        
        tableArgs = TableArgs()
        header = ["#","Profile","Role Name","Role ARN"]
        prettyTable = PrettyTable(header)
        for idx, r in enumerate(listRoles):
            columns = [ idx+1, r['profile'], r['role-name'], r['role-arn']]
            prettyTable.addRow(columns)
        prettyTable.sortByColumn(int(tableArgs.sortCol) - 1)
        prettyTable.ascendingOrder(not tableArgs.desc)
        output = "\n" + Style.RESET + prettyTable.printMe("listRoles",False,tableArgs)
        Messages.showMessage("ROLES", output)


from typing import Tuple
from awsee.sessioninfo import CallerIdentity
from awsee.messages import Messages
import boto3
from awsee.general import *
from dateutil import tz
from datetime import timedelta, tzinfo
from awsee.logmanager import LogManager
from awsee.style import Style
from awsee.utils import Utils
from botocore.exceptions import ClientError


class AwsServices:

    def __init__(self):
        pass

    def botoSession(self, profile):
        if not profile:
           return boto3.Session()
        return boto3.Session(profile_name=profile)

    def getAccountOwner(self, profile=None):
        sts = self.botoSession(profile).client("sts")
        return sts.get_caller_identity()

    def getMFADevices(self, profile=None):
        iam = self.botoSession(profile).client("iam")
        return iam.list_mfa_devices()

    def botoSessionWithToken(self, stsToken):
        session = boto3.session.Session(
            aws_access_key_id=stsToken['Credentials']['AccessKeyId'],
            aws_secret_access_key=stsToken['Credentials']['SecretAccessKey'],
            aws_session_token=stsToken['Credentials']['SessionToken']
        )
        return session
    
    def startSessionWithTokenAndAssumeRole(self, _stsToken, roleArn) -> Tuple[dict, dict]:
        stsToken = self.botoSessionWithToken(_stsToken).client('sts').assume_role(
            RoleArn=roleArn,
            RoleSessionName="mysession",
            DurationSeconds=3600
        )
        callerIdentity = self.getCallerIdentity(stsToken)
        self._convertsUTCDateExpirationToLocaZoneDate(stsToken)
        return stsToken, callerIdentity


    def assumeRole(self, roleArn, profile=None, mfaSerial=None, mfaToken=None) -> Tuple[dict, dict]:
        try:
            stsToken       = None
            callerIdentity = None
            if mfaSerial and mfaToken:
                # In case a MFA Token is necessary to assume the role
                stsToken = self.botoSession(profile).client('sts').assume_role(
                    RoleArn=roleArn,
                    RoleSessionName="mysession",
                    DurationSeconds=3600,
                    SerialNumber=mfaSerial,
                    TokenCode=mfaToken
                )
            else:
                # In case the active AWS Session does not need a MFA Role 
                stsToken = self.botoSession(profile).client('sts').assume_role(
                    RoleArn=roleArn,
                    RoleSessionName="mysession",
                    DurationSeconds=3600
                )
            callerIdentity = self.getCallerIdentity(stsToken)
        except ClientError as e:
            if e.response['Error']['Code'] == "AccessDenied":
                msg = e.response['Error']['Message']
                if not mfaSerial:
                    mfaSerial = ""
                Messages.showError("Access Denied!",f"{msg}\n    {mfaSerial}")
                return None, None
            else:
                print(e.response['Error']['Code'])
                raise e
        self._convertsUTCDateExpirationToLocaZoneDate(stsToken)
        return stsToken, callerIdentity


    def _convertsUTCDateExpirationToLocaZoneDate(self, stsToken):
        localDate = Utils.parseUTCDateToLocalZoneDate(stsToken["Credentials"]["Expiration"])
        stsToken["Credentials"]["Expiration"] = localDate

    def getCallerIdentity(self, _stsToken):
        try:
            callerIdentity = self.botoSessionWithToken(_stsToken)  \
                                  .client('sts').get_caller_identity()
            return callerIdentity
        except ClientError as e:
            if e.response['Error']['Code'] == "AccessDenied":
                msg = e.response['Error']['Message']
                Messages.showError("Access Denied!",f"{msg}")
                return None
            else:
                print(e.response['Error']['Code'])
                raise e

    def getSessionToken(self, mfaSerial, mfaToken, profile=None, durationSeconds=3600) -> Tuple[dict, dict]:
        try:
            session = self.botoSession(profile)
            stsToken = session.client('sts').get_session_token(
                DurationSeconds=durationSeconds,
                SerialNumber=mfaSerial,
                TokenCode=mfaToken,
            )
            callerIdentity = self.getCallerIdentity(stsToken)
        except ClientError as e:
            if e.response['Error']['Code'] == "AccessDenied":
                msg = e.response['Error']['Message']
                Messages.showError("Access Denied!",f"{msg}\n    {mfaSerial}")
                return None, None
            else:
                print(e.response['Error']['Code'])
                raise e
        self._convertsUTCDateExpirationToLocaZoneDate(stsToken)
        return stsToken, callerIdentity
        
        # For Temporary Tests
        # stsToken = {
        #     "AccessKeyId": "ASIAV7NRRSDFSFFGGQSP24HZ",
        #     "SecretAccessKey": "QXsks0IQyTnS9qkUntDtrukfsldhyfnwlefGB8ztFZ7",
        #     "SessionToken": "FwoGZXIvYXdzEJ3//////////wEaDDVEVIvDLPKruvdC4CKFAQ7hnQi0eWG7pjprKnsmFNAnvgMtP4foPy6KygfDcBrEExJdWxnL0S4ok=",
        #     "Expiration": datetime.datetime.now()
        # }
        # return stsToken

        
        




import re, uuid
from tinydb import Query, where
from arnparse import arnparse
from awsee.messages import Messages
from awsee.style import Style
from awsee.emoticons import Emoticons
from awsee.utils import Utils
from awsee.tableargs import TableArgs
from awsee.prettytable import PrettyTable
from awsee.repository import CredentialsRepository, MfaRepository
from awsee.general import *

class MFAManager:

    def __init__(self):
        self.credentialsRepository = CredentialsRepository()
        self.mfaRepository         = MfaRepository()

    def addMfa(self):
        Messages.showMessage("ADD MFA DEVICE TO PROFILE")
        index            = 0
        sizeSeparator    = 120
        sizeLabelProfile = 18
        output           = ""
        output           += Style.GREEN + "-".ljust(sizeSeparator,"-") + "\n"
        output           += f" {Style.IBLUE}MFA DEVICES:\n"
        profiles          = []
        for c in self.credentialsRepository.all():
            index += 1
            profiles.append(c['profile'])
            pn         = c['profile'] + " ".rjust(sizeLabelProfile -len(c['profile']), " ")
            account    = c['account']
            line       = f"  {Style.IBLUE}{index:02d} -->{Style.GREEN} {pn} {Style.IBLUE}Account..:{Style.GREEN} {account}"
            labelMfa   = "MFA Devices..: "
            mfaDevices = ""
            for idx, r in enumerate(self.mfaRepository.searchByQuery(where('profile') == c['profile'])):
                if len(mfaDevices) > 1:
                    mfaDevices += "\n" + " ".ljust(len(Utils.removeCharsColors(line)) + len(labelMfa) + 1," ")
                mfaDevices += f"{Style.BLUE}{idx+1:02d}{Style.GREEN}-{r['mfa-device']}"
            if len(mfaDevices) > 1:
                mfaDevices = f"{Style.IBLUE}{labelMfa}{Style.GREEN}{mfaDevices}{Style.GREEN}"
            output += line + f" {mfaDevices}\n"
        output += "-".ljust(sizeSeparator,"-")
        printPy(output)

        printPy(f"{Style.GREEN} {Emoticons.pin()} Choose a profile....:{Style.BLUE}", end=' ')
        result = input()
        if result == "":
           return None
           
        if not Utils.isNumber(result):
            Messages.showWarning(f"Invalid choice \"{Style.IBLUE}{result}{Style.IGREEN}\"")
            return None
        if int(result) > index:
            Messages.showWarning(f"Profile \"{Style.IBLUE}{result}{Style.IGREEN}\" not valid!")
            return None

        profile  = profiles[int(result)-1]
        printPy(f"{Style.GREEN} {Emoticons.pin()} MFA Device..........:{Style.BLUE}", end=' ')
        mfaDevice = input()

        spacesFound = re.search(" ",mfaDevice)
        if spacesFound:
            Messages.showWarning(f"Invalid MFA Device \"{Style.IBLUE}{mfaDevice}{Style.IGREEN}\", spaces are not allowed")
            return None
        try:
            arnparse(mfaDevice)
        except:
            Messages.showWarning(f"Invalid MFA Device \"{Style.IBLUE}{mfaDevice}{Style.IGREEN}\"")
            return None

        printPy(f"{Style.GREEN} {Emoticons.pin()} User Name...........:{Style.BLUE}", end=' ')
        userName = input()
        spacesFound = re.search(" ",mfaDevice)
        if spacesFound or userName == "":
            Messages.showWarning(f"Invalid User Name\"{Style.IBLUE}{mfaDevice}{Style.IGREEN}\"")
            return None

        output  = "\n"
        output += f" {Style.IBLUE} --> {Style.GREEN}Profile............: {Style.IBLUE}{profile}{Style.RESET}\n"
        output += f" {Style.IBLUE} --> {Style.GREEN}MFA Device.........: {Style.IBLUE}{mfaDevice}{Style.RESET}\n"
        output += f" {Style.IBLUE} --> {Style.GREEN}User Name..........: {Style.IBLUE}{userName}{Style.RESET}\n"
        printPy(output)

        printPy(f"{Style.GREEN} {Emoticons.pin()} Confirm [{Style.IGREEN}y/{Style.IGREEN}n{Style.RESET}]:{Style.BLUE}",end= ' ')
        confirm = input()
        if confirm.lower() == "y":
            record = {
                "id": str(uuid.uuid4()),
                "profile": profile,
                "mfa-device": mfaDevice,
                "user-name": userName
            }
            self.mfaRepository.insert(record)
            output = f"{Style.GREEN} {Emoticons.ok()} Saved!{Style.RESET}"
            printPy(output)
    
    def removeMFADevice(self, numberInList):
        mfaToRemove = None
        if numberInList > 0:
            for idx,mfa in enumerate(self.mfaRepository.all()):
                if (idx+1) == numberInList:
                    mfaToRemove = mfa
                    break
        if not mfaToRemove:
            Messages.showWarning("MFA Device not found!")
        else:
            output  = "\n"
            output += f" {Style.IBLUE} --> {Style.GREEN}Profile............: {Style.IBLUE}{mfaToRemove['profile']}{Style.RESET}\n"
            output += f" {Style.IBLUE} --> {Style.GREEN}MFA Device.........: {Style.IBLUE}{mfaToRemove['mfa-device']}{Style.RESET}\n"
            printPy(output)

            msgConfirm = f"{Style.GREEN} {Emoticons.pin()} Remove [{Style.IGREEN}y/{Style.IGREEN}n{Style.RESET}]: {Style.BLUE}"
            printPy(msgConfirm, end=" ")
            confirm = input()
            if confirm.lower() == "y":
                self.mfaRepository.remove(Query().id == mfaToRemove['id'])
                output  = "\n"
                output += f"{Style.GREEN} {Emoticons.ok()} Removed!{Style.RESET}"
                printPy(output)
    
    def listmfaDevices(self, filterByProfile):
        listmfaDevices = None
        if filterByProfile:
            listmfaDevices = self.mfaRepository.searchByQuery(where('profile') == filterByProfile)
        else:
            listmfaDevices = self.mfaRepository.all()

        if len(listmfaDevices) < 1:
            if filterByProfile:
               msg = f"There's no MFA Device added to the profile {Style.IBLUE}{filterByProfile}{Style.RESET}"
            else:
               msg = f"There's no MFA Device added" 
            Messages.showWarning(msg)
            return None
        
        tableArgs = TableArgs()
        header = ["#","Profile","MFA Device", "User Name"]
        prettyTable = PrettyTable(header)
        for idx, r in enumerate(listmfaDevices):
            columns = [ idx+1, r['profile'], r['mfa-device'], r['user-name']]
            prettyTable.addRow(columns)
        prettyTable.sortByColumn(int(tableArgs.sortCol) - 1)
        prettyTable.ascendingOrder(not tableArgs.desc)
        output = "\n" + Style.RESET + prettyTable.printMe("listMfaDevices",False,tableArgs)
        Messages.showMessage("MFA Devices", output)


import os
from os.path import expanduser
from tinydb import TinyDB, Query, where
from tinydb.storages import JSONStorage


class Repository:

    CREDENTIALS         = "CREDENTIALS"
    CONFIG              = "CONFIG"
    ROLE                = "ROLE"
    DB_CREDENTIALS_FILE = os.path.join(expanduser("~"),".awsee","cred")
    DB_CONFIG_FILE      = os.path.join(expanduser("~"),".awsee","cfg")
    DB_ROLE_FILE        = os.path.join(expanduser("~"),".awsee","role")
    DB_MFA_FILE         = os.path.join(expanduser("~"),".awsee","mfa")

    def __init__(self, _dbFile):
        self.database = TinyDB(_dbFile, indent=4)
    
    def isEmpty(self):
        return not len(self.database.all()) > 0
    
    def insert(self, record):
        self.database.insert(record)
    
    def remove(self, query):
        self.database.remove(query)

    def findByProfile(self, profile):
        return self.database.get(Query().profile == profile)
    
    def searchByQuery(self, query):
        return self.database.search(query)
    
    def update(self, record, profile):
        self.database.update(record, Query().profile == profile)
    
    def all(self):
        return self.database.all()
    
    def purge(self):
        self.database.truncate()

class CredentialsRepository(Repository):
     def __init__(self):
         super().__init__(Repository.DB_CREDENTIALS_FILE)

class ConfigRepository(Repository):
     def __init__(self):
         super().__init__(Repository.DB_CONFIG_FILE)

class RoleRepository(Repository):
     def __init__(self):
         super().__init__(Repository.DB_ROLE_FILE)

class MfaRepository(Repository):
     def __init__(self):
         super().__init__(Repository.DB_MFA_FILE)







from winreg import *
import os, sys, win32gui, win32con

class WindowsEnv:
    """     Usage:
            windowsEnv = WindowsEnv()
            subprocess.call(["cmd"], shell=True)
    """
    def __init__(self):
        try:
            path = r'SYSTEM\CurrentControlSet\Control\Session Manager\Environment'
            reg = ConnectRegistry(None, HKEY_LOCAL_MACHINE)
            key = OpenKey(reg, path, 0, KEY_ALL_ACCESS)
            
            # if len(sys.argv) == 1:
            #     self.show(key)
            # else:
            #     name, value = sys.argv[1].split('=')
            #     if name.upper() == 'PATH':
            #         value = self.queryValue(key, name) + ';' + value
            #     if value:
            #         SetValueEx(key, name, 0, REG_EXPAND_SZ, value)
            #     else:
            #         DeleteValue(key, name)

            #self.show("VBOX_MSI_INSTALL_PATH")
            #self.show(key)

            #SetValueEx(key, "XXX_JUNIOR", 0, REG_EXPAND_SZ, "UALTER")
                
            win32gui.SendMessage(win32con.HWND_BROADCAST, win32con.WM_SETTINGCHANGE, 0, 'Environment')
                                
        except Exception as e:
            raise e
    
    def show(self, key):
        for i in range(1024):                                           
            try:
                n,v,t = EnumValue(key, i)
                print('%s=%s' % (n, v))
            except EnvironmentError:
                break 
    
    def queryValue(self, key, name):       
        value, type_id = QueryValueEx(key, name)
        return value


from time import sleep
erase = '\033[A'
#erase = '\x1b[2K'

def download(number):
    print(erase + "File {} processed".format(number))

def completed(percent):
    print("({:1.1}% Completed)".format(percent))

def main():
    print("Question 1:", end=" ")
    resul = input()
    print(erase + "Question 2:", end=" ")
    resul = input()
    # for i in range(1,4):
    #     download(i)
    #     completed(i/10)
    #     sleep(1)
    

if __name__ == '__main__':
    main()
    


# coding=utf-8
#
# Author: Ualter Otoni Pereira
# ualter.junior@gmail.com
#
import os
import logging
import sys
import re
import atexit 
from os.path import expanduser
import signal
import datetime, uuid
from datetime import timezone

from shutil import which
from boto3.session import Session
from tinydb import Query, where
from arnparse import arnparse
from awsee.messages import Messages
from awsee.style import Style
from awsee.emoticons import Emoticons
from awsee.utils import Utils
from awsee.tableargs import TableArgs
from awsee.prettytable import PrettyTable
from awsee.awsservices import AwsServices
from awsee.logmanager import LogManager
from awsee.preferences import Preferences
from awsee.functions import Functions
from awsee.rolemanager import RoleManager
from awsee.mfamanager import MFAManager
from awsee.repository import Repository, CredentialsRepository, ConfigRepository, RoleRepository, MfaRepository
from awsee.sessionmanager import SessionManager
from awsee.profilemanager import ProfileManager
from awsee.awseestate import AwsSeeState
from awsee.sessioninfo import SessionInfo
from awsee.general import *

LOG = logging.getLogger("app." + __name__)
LOG.setLevel(logging.DEBUG)
FILE_LOG = os.path.join(expanduser("~"),".awsee","log")

signal.signal(signal.SIGINT, signal.SIG_DFL)

class AwsSeePy:

    def __init__(self):
        myhomeFolder = os.path.join(expanduser("~"),".awsee")
        if not os.path.exists(myhomeFolder):
           os.makedirs(myhomeFolder)

        if not os.path.exists(TEMP_CACHE_FOLDER):
            os.makedirs(TEMP_CACHE_FOLDER)

        self.mfaToken = None

        #LogManager().LOG.info("Starting...")
        self.credentialsRepository = CredentialsRepository()
        self.configRepository      = ConfigRepository()
        self.roleRepository        = RoleRepository()
        self.mfaRepository         = MfaRepository()
        self.preferences           = Preferences()
        self.executeFunction       = None
        self.myPidFile             = None


    def loadConfigProfiles(self):
        fileConfig  = expanduser("~") + "/.aws/config"

        if not os.path.exists(fileConfig):
            LogManager().LOG.warning("No file {} was found".format(fileConfig))
            return None
       
        configProfiles           = {}
        configProfiles["config"] = []
        currentProfileName = None 
        keys               = {}
        with open(fileConfig,'r') as fcfg:
            for line in fcfg.readlines():
                line = line.lstrip().rstrip()
                if line.lstrip().rstrip().startswith("["):
                   if currentProfileName:
                       configProfiles["config"].append({
                           "profile": currentProfileName,
                           "configurations": keys
                       })
                       keys = {}
                   # Read Current    
                   currentProfileName = Utils.cleanBreakLines(line).replace("[","").replace("]","")     
                else:
                   if Utils.cleanBreakLines(line).lstrip().rstrip() != "":
                      line = Utils.cleanBreakLines(re.sub('\s',"",line))
                      kv   = line.split("=")
                      keys[kv[0]] = kv[1]
            if currentProfileName:
               configProfiles["config"].append({
                   "profile": currentProfileName,
                   "configurations": keys
               })

        # Check if needs updates/inserts
        for c in configProfiles["config"]:
            dbRecordProfile = self.configRepository.findByProfile(c["profile"])
            # Insert the new one
            if not dbRecordProfile:
                self.configRepository.insert(c)
            else:
                # Update if needed
                update = False
                for config in c["configurations"]:
                    if config not in dbRecordProfile or \
                       c["configurations"][config] != dbRecordProfile["configurations"][config]:
                       update = True
                if update:
                   self.configRepository.update(c, c["profile"])

    def loadProfiles(self):
        fileCredentials = expanduser("~") + "/.aws/credentials"

        if not os.path.exists(fileCredentials):
            LogManager().LOG.warning("No file {} was found".format(fileCredentials))
            Messages.showWarning("Credentials Not Found!",f" The file {Style.IBLUE}/.aws/credentials{Style.GREEN} was not found, did you install and configure your {Style.IBLUE}awscli{Style.GREEN}?")
            sys.exit()

        currentProfileName                   = None
        accessKey                            = None
        secretKey                            = None
        credentialsProfiles                  = {}
        credentialsProfiles["credentials"]   = []
        with open(fileCredentials,'r') as fcred:
            for line in fcred.readlines():
                if line.lstrip().rstrip().startswith("["):
                   if currentProfileName:
                       defaultProfile = False
                       if currentProfileName == self.preferences.defaultProfile:
                           defaultProfile = True
                       # SAVE Previous
                       credentialsProfiles["credentials"].append({
                           "profile": currentProfileName,
                           "accessKey": accessKey,
                           "secretKey": secretKey,
                           "default": defaultProfile
                       })
                   # Read Current    
                   currentProfileName = Utils.cleanBreakLines(line).replace("[","").replace("]","") 
                else:
                   if Utils.cleanBreakLines(line).lstrip().rstrip() != "":
                      line = Utils.cleanBreakLines(re.sub('\s',"",line))
                      kv   = line.split("=")
                      if   "aws_access_key_id" in kv[0].lower():
                         accessKey = kv[1]
                      elif "aws_secret_access_key" in kv[0].lower():
                         secretKey = kv[1]
            if currentProfileName:
                defaultProfile = False
                if currentProfileName == self.preferences.defaultProfile:
                    defaultProfile = True
                # SAVE Previous
                credentialsProfiles["credentials"].append({
                    "profile": currentProfileName,
                    "accessKey": accessKey,
                    "secretKey": secretKey,
                    "default": defaultProfile
                })            

        # Check if needs updates/inserts
        awsServices = AwsServices()
        for c in credentialsProfiles["credentials"]:
            dbRecordProfile = self.credentialsRepository.findByProfile(c["profile"])
            # Insert the new one
            if not dbRecordProfile:
                account   = ""
                # Read Account and MFA Devices of the Profile at AWS
                try:
                    account = awsServices.getAccountOwner(c["profile"])["Account"]
                    for m in awsServices.getMFADevices(c["profile"])["MFADevices"]:
                        mfaRecord = self.mfaRepository.searchByQuery(Query().mfaDevice == m["SerialNumber"])
                        if not mfaRecord or len(mfaRecord) < 0:
                            self.mfaRepository.insert({
                                "profile": c["profile"],
                                "mfa-device": m["SerialNumber"],
                                "user-name": m["UserName"]
                            })
                except:
                    LogManager().LOG.warning(f"Not able to retrieve the MFA Devices for the {c['profile']}, probably some police might be blocking by the lack of MFA Token")
                c["account"] = account
                self.credentialsRepository.insert(c)
            else:
                # Check if needs to update the record (credentials change)
                if dbRecordProfile["accessKey"] != c["accessKey"]  or \
                   dbRecordProfile["secretKey"] != c["secretKey"]  or \
                   dbRecordProfile["default"]   != c["default"]:
                   # UPDATEIT
                   self.credentialsRepository.update(c, c["profile"])
                   
                # Try to Update its MFA Devices 
                # Removed to avoid remote connect all the time when the command --list is called
                #  TODO: Check to create a new function later, like --update-mfa-devices to explicitly update 
                #  the local credentials with MFA Devices remotely (if not blocked by polices)
                refreshMFA = False
                if refreshMFA:
                    try:
                        for m in awsServices.getMFADevices(c["profile"])["MFADevices"]:
                            mfaRecord = self.mfaRepository.searchByQuery(where('mfa-device') == m["SerialNumber"])
                            # Add a new MFA Device found at his/her Account (not in the MFA Device Repository yet)
                            if not mfaRecord or len(mfaRecord) < 0:
                                self.mfaRepository.insert({
                                    "profile": c["profile"],
                                    "mfa-device": m["SerialNumber"],
                                    "user-name": m["UserName"]
                                })
                    except:
                        pass

    def synchronize(self):
        printPy("")
        Messages.showStartExecution("Wait, synchronizing...")
        self.credentialsRepository.purge()
        self.configRepository.purge()
        self.loadProfiles()
        self.loadConfigProfiles()
        Messages.showStartExecution("Synchronization done!             ")
        printPy("")
    
    def listProfiles(self, more):
        profileManager = ProfileManager()
        profileManager.listProfiles(more)

    def addRole(self):
        roleManager = RoleManager()
        roleManager.addRole()

    def listRoles(self, filterByProfile):
        roleManager = RoleManager()
        roleManager.listRoles(filterByProfile)

    def removeRole(self, numberInList, roleName):
        roleManager = RoleManager()
        roleManager.removeRole(numberInList, roleName)
    
    def addMFADevice(self):
        mfaManager = MFAManager()
        mfaManager.addMfa()

    def listMFADevices(self, filterByProfile):
        mfaManager = MFAManager()
        mfaManager.listmfaDevices(filterByProfile)

    def removeMFADevice(self, numberInList):
        mfaManager = MFAManager()
        mfaManager.removeMFADevice(numberInList)

    def assumeRole(self, assumeRoleName, nameProfile=None, mfaToken=None):
        roleRecords = self.roleRepository.searchByQuery(where('role-name') == assumeRoleName)
        if not roleRecords or len(roleRecords) < 1:
           Messages.showWarning("Role not found!",f"     Role with the name {Style.IBLUE}{assumeRoleName}{Style.GREEN} was not found")
           sys.exit()

        role           = roleRecords[0]
        sessionManager = SessionManager()

        sessionInfo = SessionInfo()
        sessionInfo.role     = role
        sessionInfo.profile  = nameProfile
        sessionInfo.mfaToken = mfaToken

        output, sessionInfo = sessionManager.assumeRole(sessionInfo)
        if output:
           output = Utils.removeCharsColors(output) if self.preferences.noColor else output
           printPy(output)
        
        if sessionInfo and sessionInfo.hasStsToken:
           sessionManager.setCreateSessionAtLocalOS(sessionInfo, self.myPidFile) 

    def startAWSSessionWithMFAToken(self, nameProfile=None, mfaToken=None, assumeRole=None):
        if not nameProfile:
           # If not informed, use the Default Profile
           listCredentials = self.credentialsRepository.searchByQuery(where('default') == True)
           if not listCredentials or len(listCredentials) < 1:
               Messages.showWarning("Default profile not defined!")
               sys.exit()
           credentials = listCredentials[0]
        else:
            # Get the Profile using the name informed
            credentials = self.credentialsRepository.findByProfile(nameProfile)
            if not credentials:
               Messages.showWarning(f"Profile {Style.IBLUE}{nameProfile}{Style.GREEN} not defined!")
               sys.exit()

        mfa = self.mfaRepository.findByProfile(credentials['profile'])
        if not mfa:
           Messages.showWarning(f"MFA for the profile {credentials['profile']} not found!")
           sys.exit()

        role = None
        if assumeRole:
            roles = self.roleRepository.searchByQuery(where('role-name') == assumeRole)
            if not roles or len(roles) < 1:
               Messages.showWarning(f"Role {assumeRole} not found!")
               sys.exit()
            role = roles[0]

        sessionManager = SessionManager()

        sessionInfo = SessionInfo()
        sessionInfo.credentials = credentials
        sessionInfo.mfa         = mfa
        sessionInfo.role        = role
        sessionInfo.mfaToken    = mfaToken

        output, sessionInfo = sessionManager.startAWSSessionWithMFA(sessionInfo)
        if output:
           output = Utils.removeCharsColors(output) if self.preferences.noColor else output
           printPy(output)
        
        if sessionInfo and sessionInfo.hasStsToken:
           sessionManager.setCreateSessionAtLocalOS(sessionInfo, self.myPidFile)
    
    def startAWSSession(self, nameProfile, nameRole=None):
        credentials = self.credentialsRepository.findByProfile(nameProfile)
        if not credentials:
           Messages.showWarning(f"Profile {Style.IBLUE}{nameProfile}{Style.GREEN} not defined!")
           sys.exit()

        # Check if the asked profile has a MFA Device associate with it, if yes ask for a MFA Token
        mfaToken = None
        mfa      = self.mfaRepository.findByProfile(nameProfile)
        if mfa:
           label = f"\n{Style.IBLUE} ->> {Style.GREEN}The profile {Style.IBLUE}{nameProfile} {Style.GREEN}has an MFA Device associated\n\n \
    Please enter a MFA Token.......:{Style.IBLUE}"
           printPy(label, end=' ') 
           while True:
               mfaToken = input()
               if self.checkMFATokenValid(mfaToken):
                  break
               Messages.showWarning(f"Invalid Token! Try again or [{Style.IBLUE}BLANKS{Style.GREEN}] to exit...")
               printPy(f"     {Style.GREEN}Please enter a MFA Token.......:{Style.IBLUE}", end=' ')
        
        role = None
        if nameRole:
            roles = self.roleRepository.searchByQuery(where('role-name') == nameRole)
            if roles and len(roles) > 0:
                role = roles[0]

        sessionManager = SessionManager()
        if   not mfaToken and not nameRole:
           # No need for MFA Token and no Role So, add the default credentials direct in the Session
           sessionInfo = SessionInfo()
           sessionInfo.credentials = credentials
           output, sessionInfo = sessionManager.startAWSSession(sessionInfo)
        elif not mfaToken and nameRole:
           # No need for MFA, but has Role, let's assume the Role
           sessionInfo = SessionInfo()
           sessionInfo.credentials = credentials
           sessionInfo.role        = role
           #sessionInfo.profile     = credentials["profile"]
           output, sessionInfo = sessionManager.assumeRole(sessionInfo) 
        else:
           # With MFA Token Profile and Role (if it is there)
           sessionInfo = SessionInfo()
           sessionInfo.credentials = credentials
           sessionInfo.role        = role
           sessionInfo.mfaToken    = mfaToken
           sessionInfo.mfa         = mfa
           output, sessionInfo     = sessionManager.startAWSSessionWithMFA(sessionInfo) 

        if output:
           output = Utils.removeCharsColors(output) if self.preferences.noColor else output
           printPy(output)
        
        if sessionInfo and sessionInfo.stsToken:
           sessionManager.setCreateSessionAtLocalOS(sessionInfo, self.myPidFile) 
        
    def syntax(self, short=False):
        printPy("")
        functions = Functions()
        functions.showUsage(short)
        functions.showFunctions(short)
    
    def askForMFAToken(self):
        defaultProfile = self.credentialsRepository.searchByQuery(where("default") == True)
        if defaultProfile and len(defaultProfile) > 0:
            profile = defaultProfile[0]
            mfa     = self.mfaRepository.findByProfile(profile['profile'])
            if mfa:
                printPy(f"{Style.GREEN}")
                printPy(f"Profile.......: {Style.IBLUE}{profile['profile']}{Style.GREEN}")
                printPy(f"Account.......: {Style.IBLUE}{profile['account']}{Style.GREEN}")
                printPy(f"{Style.GREEN}MFA Token.....:{Style.IBLUE}", end=' ')
                while True:
                    response = input()
                    if response == "":
                        printPy("")
                        sys.exit()
                    if self.checkMFATokenValid(response):
                        break
                    Messages.showWarning(f"Invalid Token! Try again or [{Style.IBLUE}BLANKS{Style.GREEN}] to exit...")
                    printPy(f"{Style.GREEN}Profile.......: {Style.IBLUE}{profile['profile']}{Style.GREEN}")
                    printPy(f"Account.......: {Style.IBLUE}{profile['account']}{Style.GREEN}")
                    printPy(f"{Style.GREEN}MFA Token.....:{Style.IBLUE}", end=' ')
                self.startAWSSessionWithMFAToken(profile['profile'], response, None)
            else:
                self.startAWSSession(profile['profile'])
            printPy("")
            sys.exit()
        else:
            Messages.showError("Default profile not defined!")
            self.syntax()
        self.syntax()
        sys.exit()

    def bootStrap(self):
        SessionManager().createAliasFile()

    def start(self):
        #executeFunction = False
        args = sys.argv

        self.bootStrap()

        # Remove PID File if found
        for a in args:
            found = re.search("pid_[0-9]{1,}_pid",a) 
            if found:
                self.myPidFile = found.group()
                args.remove(a)

        if len(args) < 2:
           # In case no argument was informed, let's use the default profile
           # and if needed MFA Token lets request it
           self.askForMFAToken()

        # Check for -nt / --new-terminal
        if "-nt" in args or "--new-terminal" in args:
            AwsSeeState.NEW_TERMINAL = True
            # Remove this "-nt or --new-terminal" from args
            if "-nt" in args:
               args.remove("-nt")
            if "--new-terminal" in args:
               args.remove("--new-terminal")
            # in case there's nothing more, ask for the MFA Token
            if len(args) < 2:
                self.askForMFAToken()

        functions = Functions()
        if "-h" in args or "--help" in args:
            short = True if "-h" in args else False
            if len(args) > 2:
               f = functions.getFunctionByArgumentIdentifier(args[2])
               output = functions.showFunction(f)
               output = Utils.removeCharsColors(output) if self.preferences.noColor else output
               printPy(output)
            else:
               self.syntax(short)
            sys.exit()
        
        if "-u" in args or "--usage" in args:
            functions.showUsage(False)
            sys.exit()
        
        self.loadProfiles()
        self.loadConfigProfiles()

        args                 = args[1:]                          # Discard the first argument (our python function name)
        self.executeFunction = self.parseArgsFindFunction(args)  # Identify which function by the arguments/identifiers (-l --list, etc.)

        if  functions.INFO == self.executeFunction:
            self.showInfo()
        
        elif  functions.ENV == self.executeFunction:
            SessionManager().showEnvironment()
        
        elif functions.CLEAN == self.executeFunction:
            SessionManager().cleanAwsSession()
        
        elif  functions.LIST_PROFILES == self.executeFunction:
            more = True if "full" in args or "-lf" in args else False
            self.listProfiles(more)

        elif functions.SYNC == self.executeFunction:
            self.synchronize()

        elif functions.ADD_ROLE == self.executeFunction:
            self.addRole()

        elif functions.LIST_ROLES == self.executeFunction:
            filterByProfile = None
            if len(args) > 1:
               filterByProfile = args[1]
            self.listRoles(filterByProfile)

        elif functions.REMOVE_ROLE == self.executeFunction:
            numberInList = -1
            roleName     = None
            if len(args) > 1:
               if Utils.isNumber(args[1]):
                   numberInList = int(args[1])
               else:
                   roleName = args[1]
            else:
                Messages.showWarning("Missing parameter! At least one of them must be informed:")
                output = Functions().showFunction(Functions.REMOVE_ROLE)
                output = Utils.removeCharsColors(output) if self.preferences.noColor else output
                printPy(output)
                sys.exit()
            self.removeRole(numberInList, roleName)

        elif functions.ADD_MFA == self.executeFunction:
             self.addMFADevice()

        elif functions.LIST_MFAS == self.executeFunction:
             filterByProfile = None
             if len(args) > 1:
                filterByProfile = args[1]
             self.listMFADevices(filterByProfile)

        elif functions.REMOVE_MFA == self.executeFunction:
            numberInList = -1
            if len(args) > 1:
               if Utils.isNumber(args[1]):
                   numberInList = int(args[1])
            else:
                Messages.showWarning("Missing parameter for Remove MFA...")
                output = Functions().showFunction(Functions.REMOVE_MFA)
                output = Utils.removeCharsColors(output) if self.preferences.noColor else output
                printPy(output)
                sys.exit()
            self.removeMFADevice(numberInList)

        elif functions.ASSUME_ROLE == self.executeFunction:
            if len(args) > 1:
               role = args[1]
               self.assumeRole(role,None,None)
            else:
               Messages.showWarning("Missing [ROLE] parameter...")
               output = Functions().showFunction(Functions.ASSUME_ROLE)
               output = Utils.removeCharsColors(output) if self.preferences.noColor else output
               printPy(output)
               sys.exit()

        elif functions.START_SESSION == self.executeFunction:
            if len(args) > 1:
                profile = args[1]
                role    = args[3] if len(args) > 3 else None
                self.startAWSSession(nameProfile=profile, nameRole=role)
            else:
                Messages.showWarning("Missing [PROFILE] parameter...")
                output = Functions().showFunction(Functions.START_SESSION)
                output = Utils.removeCharsColors(output) if self.preferences.noColor else output
                printPy(output)
                sys.exit()

        elif functions.START_SESSION_MFA_TOKEN == self.executeFunction:
            # Parameters Already validated (if exists)
            mfaToken = args[0]
            profile  = args[2] if len(args) > 2 else None
            role     = args[4] if len(args) > 4 else None
            if not self.checkMFATokenValid(mfaToken):
                Messages.showWarning("Invalid Parameter!",f"    Is {Style.IBLUE}{mfaToken}{Style.GREEN} MFA Token?  Please, check it\n")
                sys.exit()
            self.startAWSSessionWithMFAToken(nameProfile=profile, mfaToken=mfaToken, assumeRole=role)
        else:
             self.exitWithMessageInvalidUsage()

    def parseArgsFunctionStartAWSSession(self, args):
        functions = Functions()

        # Check if a profile was passed: "-p [profile] or -p [profile] -r [role]"
        role = None
        if len(args) > 1:
           if len(args) == 4:
               self.exitWithMessageInvalidUsage()
           # Here, It should be -p [PROFILE]    
           elif len(args) == 3:
              if args[1] == "-p":
                 profile = args[2]
              elif args[1] != "-p":
                 self.exitWithMessageInvalidUsage()
           # Here, It should be -p [PROFILE] -r [ROLE]
           elif len(args) == 5:     
               if   args[1] == "-p":
                    profile = args[2]
               if   args[3] == "-p":
                    profile = args[4] 
               if args[1] == "-r":
                    role = args[2]  
               if args[3] == "-r":
                    role    = args[4]
               if (not profile or profile == "") or (not role or role == ""):
                   self.exitWithMessageInvalidUsage()
           else:
              Messages.showError(f"Missing parameter profile [-p profile], check syntax with {Style.GREEN}awsee -h{Style.RESET}")
              sys.exit()

        # Start AWS Session with Default Profile
        return functions.START_SESSION_MFA_TOKEN
    
    def parseArgsFindFunction(self, args):
        functions = Functions()

        # Check if the First Argument is a Valid MFA Token (Six-Digit)  Obs: The lack of Arguments was already tested before
        isMFAToken = self.checkMFATokenValid(args[0])
        if isMFAToken:
           self.parseArgsFunctionStartAWSSession(args)

        #
        # Looking for an auxiliar functions (Add Role, Add MFA, List Profiles, Remove Role, Sync, etc.)
        #
        for arg in args:
            func = functions.getFunctionByArgumentIdentifier(arg)
            if func:
                # Check if there's a MFA Token Informed, if it is, the funcion asked was not START_SESSION only, the argument -p must be the first one
                if func == functions.START_SESSION:
                   if args[0] == "-p":
                      return func
                   else:
                      continue
                # Check if there's a MFA Token Informed, if it is, the funcion asked was not ASSUME_ROLE, it is START_SESSION_MFA_TOKEN
                if func == functions.ASSUME_ROLE:
                   if args[0] == "-r":
                      return func
                   else:
                      continue 
                return func

        return self.parseArgsFunctionStartAWSSession(args)

        

    def showInfo(self):
        if 'AWS_SEE' in os.environ:
            if os.environ['AWS_SEE'] == "CLOSED":
                Messages.showWarning("The AWSee Session was closed already!")
                sys.exit()
        else:
            Messages.showWarning("No AWSee Session was started in this terminal!")
            sys.exit()
        
        account      = os.environ['AWS_ACCOUNT'] if 'AWS_ACCOUNT' in os.environ else None
        user         = os.environ['AWS_USER_ARN'] if 'AWS_USER_ARN' in os.environ else None
        userId       = os.environ['AWS_USER_ID'] if 'AWS_USER_ID' in os.environ else None
        accessKey    = os.environ['AWS_ACCESS_KEY_ID'] if 'AWS_ACCESS_KEY_ID' in os.environ else None
        secretKey    = os.environ['AWS_SECRET_ACCESS_KEY'] if 'AWS_SECRET_ACCESS_KEY' in os.environ else None
        expiration   = os.environ['AWS_EXPIRATION'] if 'AWS_EXPIRATION' in os.environ else None
        profile      = os.environ['AWS_PROFILE'] if 'AWS_PROFILE' in os.environ else ""
        role         = os.environ['AWS_ROLE'] if 'AWS_ROLE' in os.environ else ""
        tokenNeeded  = os.environ['AWS_WITH_TOKEN'] if 'AWS_WITH_TOKEN' in os.environ else ""

        if not account or account == "" and profile and profile != "":
            credential = self.credentialsRepository.findByProfile(profile)
            account = credential["account"]

        if expiration and expiration != "":
            expDate         = datetime.datetime.strptime(expiration,"%Y-%m-%d %H:%M:%S%z")

            if (expDate < datetime.datetime.now(timezone.utc)):
                diff  = (datetime.datetime.now(timezone.utc) - expDate).seconds / 60
                Messages.showWarning(f"Your AWS Session expired {Style.IBLUE}{diff:02.0f} {Style.IGREEN}minutes ago\n     At {Style.IMAGENTA}{expiration}")

                profiles = self.credentialsRepository.searchByQuery(where("profile") == profile)
                if profiles and len(profiles) > 0:
                    label = f"   {Style.GREEN}Would like to re-connect to profile {Style.IBLUE}{profiles[0]['profile']}"
                    if role and role != "":
                        label += f" {Style.GREEN}role {Style.IBLUE}{role}{Style.GREEN}? [y/n]{Style.IBLUE}" 
                    else:
                        label += f"{Style.GREEN}? [y/n]{Style.IBLUE}"
                    printPy(label, end=' ')
                    response = input()
                    if response.lower() == "y":
                        if tokenNeeded:
                           printPy(f"   {Style.GREEN}MFA Token.........:{Style.IBLUE}", end=' ')
                           while True:
                               response = input()
                               if response == "":
                                   printPy("")
                                   sys.exit()
                               if self.checkMFATokenValid(response):
                                  break
                               Messages.showWarning(f"Invalid Token! Try again or hit [{Style.IBLUE}ENTER{Style.GREEN}] to exit...")
                               printPy(f"   {Style.GREEN}MFA Token.........:{Style.IBLUE}", end=' ')
                           self.startAWSSessionWithMFAToken(profile,response,role if role != "" else None)
                           sys.exit()
                        else:
                           self.startAWSSession(profile)
                           sys.exit()
                    printPy("")
                sys.exit()
            diff            = (expDate - datetime.datetime.now(timezone.utc)).seconds / 60
            expirationLabel = f"{Style.GREEN}Expiration........: {Style.IBLUE}{expiration} ({Style.YELLOW}{diff:02.0f} {Style.IBLUE}minutes remaining)"
        else:
            expirationLabel = None

        output  = f"\n{Style.IMAGENTA}#awsee{Style.GREEN}\n"
        output += f"=============================\n"
        output += f"      {Style.IMAGENTA}My AWSee Session{Style.GREEN}\n"
        output += f"=============================\n"
        if account:
            output += f"{Style.GREEN}Account...........: {Style.IBLUE}{account}\n"
        if user:
            output += f"{Style.GREEN}User..............: {Style.IBLUE}{user}\n"
        if userId:
            output += f"{Style.GREEN}User Id...........: {Style.IBLUE}{userId}\n"
        if profile:
            output += f"{Style.GREEN}Profile...........: {Style.IBLUE}{profile}\n"
        if role:
            output += f"{Style.GREEN}Role..............: {Style.IBLUE}{role}\n"
        if accessKey:
            output += f"{Style.GREEN}Access Key........: {Style.IBLUE}{accessKey}\n"
        if secretKey:
            secretKey = secretKey[:10] + "*".ljust(10,"*")
            output += f"{Style.GREEN}Secret Key........: {Style.IBLUE}{secretKey}\n"
        if expirationLabel:
            output += f"\n{Emoticons.pointRight()} {Style.GREEN}{Style.IBLUE}Your AWS Session will expires in {Style.YELLOW}{diff:02.0f} {Style.IBLUE}minutes\n{Emoticons.pointRight()} At {Style.IMAGENTA}{expiration}{Style.RESET}\n"
        output = Utils.removeCharsColors(output) if self.preferences.noColor else output
        printPy( output )

    def exitWithMessageInvalidUsage(self):
        Messages.showError(f"Invalid usage!\n     Please, check syntax with {Style.GREEN}awsee -h{Style.RESET}")
        sys.exit() 

    def checkMFATokenValid(self, token):
        f = re.findall("[0-9]{6}$",token)
        if len(f) > 0:
            return True
        return False

@atexit.register 
def goodbye(): 
    pass

def main():
    """Start AwsSe"""
    try:
        awsee = AwsSeePy()
        awsee.start()
    except KeyboardInterrupt:
        pass    
    except UnicodeEncodeError as e:
        if Utils.isWindows():
           output = f"\n{Style.IRED} --> {Style.IMAGENTA}UnicodeEncode Error{Style.GREEN}\n     If you are using GitBash at Windows\n     disable the use of emoticons at ~/.awsse/awsee.ini, {Style.IBLUE}emoticons-enabled = false{Style.RESET}\n\n"
           output = Utils.removeCharsColors(output) if Preferences().noColor else output
           printPy(output)

if __name__ == '__main__':
    main()







import sys, os
from os.path import expanduser


TEMP_CACHE_FOLDER = os.path.join(expanduser("~"),".awsee","tmp")

def printPy(message: str, end: str=None):
    readyMsg = str(message)
    if end:
       print(readyMsg, file=sys.stderr, end=end)
    else:
       print(readyMsg, file=sys.stderr)

def printOut(message: str):
    readyMsg = str(message)
    print(readyMsg)
    



import json, datetime, os
import clipboard
import pytz
from io import StringIO
from dateutil import tz
from os import stat
from pygments import highlight, lexers, formatters
from awsee.style import Style


class Utils:
    
    @staticmethod
    def formatPrettyJson(dictResult):
        jsonResult = Utils.dictToJson(dictResult)
        colorful_json = highlight(jsonResult, lexers.JsonLexer(), formatters.TerminalFormatter())
        return colorful_json
    
    @staticmethod 
    def dictToJson(response, sortKeys=False):
        strObj = StringIO()
        json.dump(response, strObj, default=str, indent=4, sort_keys=sortKeys)
        return strObj.getvalue()
    
    @staticmethod 
    def cleanBreakLines(text):
        if isinstance(text, str):
           text = text.replace("\n","")
           return text
        return text
    
    @staticmethod
    def addToClipboard(text):
        try:
            textNoColor = Utils.removeCharsColors(text)
            if textNoColor:
               clipboard.copy(textNoColor.strip())
        except:
            pass
    
    @staticmethod
    def isRunningOnGitBash():
        try:
            # Probably on a GitBash environment (Windows)
            os.environ["SHELL"]
            return True
        except:
            return False

    @staticmethod
    def isWindows():
        return 1 if os.name == "nt" else 0
    
    @staticmethod
    def parseUTCDateToLocalZoneDate(utc_date):
        #utc_date       = datetime.datetime.strptime(string_date, "%Y-%m-%d %H:%M:%S+00:00")
        local_timezone = tz.tzlocal() 
        local_datetime = utc_date.replace(tzinfo=pytz.utc)
        local_datetime = local_datetime.astimezone(local_timezone)
        return local_datetime
        

    @staticmethod
    def isNumber(s):
        try:
            int(s)
            return True
        except ValueError:
            return False
    
    @staticmethod
    def removeCharsColors(text):
        if isinstance(text, str):
           text = text.replace(Style.UNDERLINE,"") \
                      .replace(Style.RESET,"") \
                      .replace(Style.BLACK,"") \
                      .replace(Style.RED,"") \
                      .replace(Style.GREEN,"") \
                      .replace(Style.YELLOW,"") \
                      .replace(Style.BLUE,"") \
                      .replace(Style.MAGENTA,"") \
                      .replace(Style.CYAN,"") \
                      .replace(Style.WHITE,"") \
                      .replace(Style.BBLACK,"") \
                      .replace(Style.BRED,"") \
                      .replace(Style.BGREEN,"") \
                      .replace(Style.BYELLOW,"") \
                      .replace(Style.BBLUE,"") \
                      .replace(Style.BMAGENTA,"") \
                      .replace(Style.BCYAN,"") \
                      .replace(Style.BWHITE,"") \
                      .replace(Style.BG_BLACK,"") \
                      .replace(Style.BG_RED,"") \
                      .replace(Style.BG_GREEN,"") \
                      .replace(Style.BG_YELLOW,"") \
                      .replace(Style.BG_BLUE,"") \
                      .replace(Style.BG_PURPLE,"") \
                      .replace(Style.BG_CYAN,"") \
                      .replace(Style.BG_WHITE,"") \
                      .replace(Style.IBLACK,"") \
                      .replace(Style.IRED,"") \
                      .replace(Style.IGREEN,"") \
                      .replace(Style.IYELLOW,"") \
                      .replace(Style.IBLUE,"") \
                      .replace(Style.IMAGENTA,"") \
                      .replace(Style.ICYAN,"") \
                      .replace(Style.IWHITE,"") \
                      .replace(Style.BIBLACK,"") \
                      .replace(Style.BIRED,"") \
                      .replace(Style.BIGREEN,"") \
                      .replace(Style.BIYELLOW,"") \
                      .replace(Style.BIBLUE,"") \
                      .replace(Style.BIPURPLE,"") \
                      .replace(Style.BICYAN,"") \
                      .replace(Style.BIWHITE,"") \
                      .replace(Style.On_IBLACK,"") \
                      .replace(Style.On_IRED,"") \
                      .replace(Style.On_IGREEN,"") \
                      .replace(Style.On_IYELLOW,"") \
                      .replace(Style.On_IBLUE,"") \
                      .replace(Style.On_IPURPLE,"") \
                      .replace(Style.On_ICYAN,"") \
                      .replace(Style.On_IWHITE,"")
        return text



# coding=utf-8
#
# Author: Ualter Otoni Pereira
# ualter.junior@gmail.com
#
import os
import logging
import sys
import re, uuid
import atexit 
from os.path import expanduser
import signal, subprocess

from shutil import which
from tinydb import Query, where
from arnparse import arnparse
from awsee.messages import Messages
from awsee.style import Style
from awsee.emoticons import Emoticons
from awsee.utils import Utils
from awsee.tableargs import TableArgs
from awsee.prettytable import PrettyTable
from awsee.awsservices import AwsServices
from awsee.logmanager import LogManager
from awsee.preferences import Preferences
from awsee.functions import Functions
from awsee.rolemanager import RoleManager
from awsee.mfamanager import MFAManager
from awsee.repository import Repository, CredentialsRepository, ConfigRepository, RoleRepository, MfaRepository
from awsee.sessionmanager import SessionManager
from awsee.general import *

class ProfileManager:

    def __init__(self):
        self.credentialsRepository = CredentialsRepository()
        self.configRepository      = ConfigRepository()
        self.roleRepository        = RoleRepository()
        self.mfaRepository         = MfaRepository()
        self.preferences           = Preferences()

    def listProfiles(self, more=True):
        # defaultProfileAccessKey = ""
        # defaultCredential       = self.credentialsRepository.searchByQuery(where('default') == True)
        # if defaultCredential and len(defaultCredential) > 0:
        #    defaultProfileAccessKey = defaultCredential[0]["accessKey"]

        tableArgs = TableArgs()
        # Hightlight the Default Account
        #tableArgs.setArguments("| " + defaultProfileAccessKey)
        header = ["#","Profile","Access Key", "Secret Key", "Region", "Account", "MFA Serial"]
        if more:
            header.append("Role Name")
            header.append("Role Arn")
        prettyTable = PrettyTable(header)

        idx = 0
        for credential in self.credentialsRepository.all():
            idx += 1

            profileName   = credential['profile']
            account       = str(credential["account"])
            accessKey     = credential['accessKey']
            secretKey     = "*".ljust(5,"*") + credential['secretKey'][:5]
            region        = ""
            configProfile = self.searchConfigForProfile(profileName)
            if configProfile and "region" in configProfile:
               region = configProfile["region"]


            mfaDevices = self.mfaRepository.searchByQuery(where('profile') == profileName)
            addSeparator  = False
            if more:
                listRoles = self.roleRepository.searchByQuery(Query().profile == profileName)
                if len(listRoles) > 0:
                    addSeparator = True
                    if len(mfaDevices) > len(listRoles):
                        for idxMfa, mfaDevice in enumerate(mfaDevices):
                            mfa      = f"{mfaDevice['mfa-device']}"
                            roleName = ""
                            roleArn  = ""
                            if (idxMfa+1) <= len(listRoles):
                                roleName = listRoles[idxMfa]['role-name']
                                roleArn  = listRoles[idxMfa]['role-arn']
                            columns = [ idx, profileName, accessKey, secretKey, region, account, mfa, roleArn, roleName]
                            prettyTable.addRow(columns)
                    else:    
                        for idxRole, role in enumerate(listRoles):
                            mfa      = ""
                            roleName = role['role-name']
                            roleArn  = role['role-arn']
                            if (idxRole+1) <= len(mfaDevices):
                               mfa  = f"{mfaDevices[idxRole]['mfa-device']}" 
                            columns = [ idx, profileName, accessKey, secretKey, region, account, mfa, roleArn, roleName]
                            prettyTable.addRow(columns)
                else:
                    if len(mfaDevices) > 0:
                        addSeparator = True if len(mfaDevices) > 1 else False
                        for idxMfa, mfaDevice in enumerate(mfaDevices):
                            mfa  = f"{mfaDevice['mfa-device']}"
                            columns = [ idx, profileName, accessKey, secretKey, region, account, mfa, "", ""]
                            prettyTable.addRow(columns)
                    else:
                        columns = [ idx, profileName, accessKey, secretKey, region, account, "", "", ""]
                        prettyTable.addRow(columns)
                if addSeparator:
                    prettyTable.addSeparatorGroup()    
            else:
                if len(mfaDevices) > 0:
                    addSeparator = True if len(mfaDevices) > 1 else False
                    for idxMfa, mfaDevice in enumerate(mfaDevices):
                        mfa  = f"{mfaDevice['mfa-device']}"
                        columns = [ idx, profileName, accessKey, secretKey, region, account, mfa]
                        prettyTable.addRow(columns)
                else:
                    addSeparator = False
                    columns = [ idx, profileName, accessKey, secretKey, region, account, ""]
                    prettyTable.addRow(columns)
                if addSeparator:
                    prettyTable.addSeparatorGroup()

        prettyTable.sortByColumn(int(tableArgs.sortCol) - 1)
        prettyTable.ascendingOrder(not tableArgs.desc)
        output = "\n" + Style.RESET + prettyTable.printMe("listProfiles",False,tableArgs)
        Messages.showMessage("PROFILES", output)
    
    def searchConfigForProfile(self, profileName):
        profileConfigurations = self.configRepository.findByProfile(profileName)
        return profileConfigurations["configurations"] if profileConfigurations else None
        


import os, sys
import re

class TableArgs:

    def __init__(self):
        self.sortCol            = "1"
        self.desc               = False
        self.tagsTemp           = {}
        self.showTags           = False
        self.verbose            = False
        self.excelFile          = False
        self.csvDrawIo          = False
        self.saveToFile         = False
        self.highlightValue     = None
        self.filterValue        = None
        self.commandArguments   = None

    def setArguments(self, _commandArguments):
        self.commandArguments = _commandArguments

        arguments = self.commandArguments.split(",")

        for arg in arguments:
            if "sort" in arg:
                self.sortCol = arg.replace("sort","")  
            if "desc" in arg:
                self.desc = True
            if "showtags" in arg:
                self.showTags = True
            if "excel" in arg:
                self.excelFile = True
            if "drawio" in arg:
                self.csvDrawIo = True
            if "verbose" in arg:
                self.verbose = True
            if "save" in arg:
                self.saveToFile = True            
            # Look for  tags:Enviroment=Production;Project=Ecommerce    
            #if ("tags" in arg or "tag" in arg) and arg != "showtags":
            if  arg.startswith("tag:") or arg.startswith("tags:"):
                tagsArg = arg.replace("tags","tag").replace("tag:","")
                if "#" in tagsArg:
                  for tag in tagsArg.split("#"):
                     tagName  = tag.split("=")[0]
                     tagValue = tag.split("=")[1]
                     self.tagsTemp[tagName] = self._cleanPipelineArguments(tagValue)
                else:
                     tagName  = tagsArg.split("=")[0]
                     tagValue = tagsArg.split("=")[1]
                     self.tagsTemp[tagName] = self._cleanPipelineArguments(tagValue)


        if "|" in self.commandArguments:
           pos        = self.commandArguments.find("|")
           complement = self.commandArguments[pos:].replace(" | ","").replace("| ","").replace(" |","")
           if complement.startswith("grep"):
              self.filterValue = complement.replace("grep","") 
           else:
              self.highlightValue = complement   
           
        
        #print("verbose:     " + str(self.verbose))
        #print("saveToFile:  " + str(self.saveToFile))
        #print("desc:        " + str(self.desc))
        #if self.highlightValue: print("highlightValue:  " + self.highlightValue)
        #if self.filterValue: print("filterValue:  " + self.filterValue)
        #sys.exit()

    def cleanPipelineArguments(self):
        return self._cleanPipelineArguments(self.commandArguments)

    def _cleanPipelineArguments(self, str):
        if "|" in str:
            return str[:str.find("|")].strip()
        else:    
            return str




from awsee import general


class SessionInfo(object):

    def __init__(self):
        self._credentials    = None
        self._role           = None
        self._mfa            = None
        self._callerIdentity = None
        self._isBashEnv      = False
        self._scriptFileWin  = None
        self._scriptFileBash = None
        self._stsToken       = None
        self._mfaToken       = None
        self._uuid           = None
        self._config         = None

    @property
    def isBashEnv(self):
        return self._isBashEnv
    @isBashEnv.setter
    def setBashEnv(self, value):
        self._isBashEnv = value

    @property
    def scriptFile(self):
        if self._isBashEnv:
            return self._scriptFileBash
        return self._scriptFileWin
    
    @property
    def scriptFileWin(self):
        return self._scriptFileWin
    @scriptFileWin.setter
    def scriptFileWin(self, value):
        self._scriptFileWin = value

    @property
    def scriptFileBash(self):
        return self._scriptFileBash
    @scriptFileBash.setter
    def scriptFileBash(self, value):
        self._scriptFileBash = value
    
    @property
    def accessKey(self, returnOpt=None):
        return self.stsToken["AccessKeyId"] \
               if self.stsToken else returnOpt
    
    @property
    def secretKey(self, returnOpt=None):
        return self.stsToken["SecretAccessKey"] \
               if self.stsToken else returnOpt
    
    @property
    def hasSessionToken(self):
        return True if self._stsToken and "SessionToken" in self._stsToken else True
    @property
    def sessionToken(self, returnOpt=None):
        return self.stsToken["SessionToken"] \
               if self.stsToken else returnOpt

    @property
    def hasExpiration(self):
        return True if self._stsToken and "Expiration" in self._stsToken else False
    @property
    def expiration(self, returnOpt=None):
        return self.stsToken["Expiration"] \
               if self.stsToken else returnOpt
    
    @property
    def profile(self):
        return self._credentials["profile"]
    @profile.setter
    def profile(self, value):
        self._credentials = {}
        self._credentials["profile"] = value
    @property
    def hasProfile(self):
        return True if self._credentials and "profile" in self._credentials else False
    
    @property
    def role(self):
        return self._role
    @role.setter
    def role(self, value):
        self._role = value
    @property
    def hasRole(self):
        return True if self._role else False
    @property
    def roleName(self, returnOpt=None):
        return self._role['role-name'] \
               if self._role and 'role-name' in self._role else \
               returnOpt
    @property
    def roleArn(self, returnOpt=None):
        return self._role['role-arn'] \
               if self._role and 'role-arn' in self._role else \
               returnOpt
    @property
    def roleProfile(self, returnOpt=None):
        return self._role['profile'] \
               if self._role and 'profile' in self._role else \
               returnOpt
    
    @property
    def stsToken(self):
        return self._stsToken
    @stsToken.setter
    def stsToken(self, value):
        self._stsToken = value
    @property
    def hasStsToken(self):
        return True if self._stsToken else False
    
    @property
    def mfaToken(self):
        return self._mfaToken
    @mfaToken.setter
    def mfaToken(self, value):
        self._mfaToken = value
    @property
    def hasMfaToken(self):
        return True if self._mfaToken else False
    
    @property
    def config(self):
        return self._config
    @config.setter
    def config(self, value):
        self._config = value
    @property
    def hasConfig(self):
        return True if self._config else False
    
    @property
    def mfa(self):
        return self._mfa
    @mfa.setter
    def mfa(self, value):
        self._mfa = value
    @property
    def mfaDevice(self, returnOpt=None):
        return self._mfa['mfa-device'] \
               if self._mfa and 'mfa-device' in self._mfa else \
               returnOpt
    
    @property
    def uuid(self):
        return self._uuid
    @uuid.setter
    def uuid(self, value):
        self._uuid = value
    
    @property
    def credentials(self):
        return self._credentials
    @credentials.setter
    def credentials(self, value):
        self._credentials = value
    
    @property
    def callerIdentity(self):
        return self._callerIdentity
    @callerIdentity.setter
    def callerIdentity(self, value):
        self._callerIdentity = CallerIdentity(value)

    
class CallerIdentity:
    def __init__(self, callerIdentity):
        self._userId  = callerIdentity['UserId']
        self._account = callerIdentity['Account']
        self._arn     = callerIdentity['Arn']
    
    @property
    def userId(self):
        return self._userId
    @property
    def account(self):
        return self._account
    @property
    def arn(self):
        return self._arn
    


from awsee.style import Style
from awsee.emoticons import Emoticons
from awsee.preferences import Preferences
from awsee.utils import Utils
from awsee.general import *

class Messages:

    def __init__(self):
        pass

    @staticmethod
    def formatMessage(title,msg=None):
        emoticon = Emoticons.pointRight()
        output = ""
        #output  = Style.ICYAN + "=".ljust(len(title)+5,"=")
        output += f"\n{Style.IBLUE} {emoticon} {Style.IGREEN}{title}{Style.RESET}\n"
        #output += Style.ICYAN + "=".ljust(len(title)+5,"=")
        if msg:
           output += f"{Style.GREEN}{msg}"
        return output
    def showMessage(title,msg=None):
        output = Messages.formatMessage(title,msg)
        output = Utils.removeCharsColors(output) if Preferences().noColor else output
        printPy(output)
    
    @staticmethod
    def formatWarning(title,msg=None):
        emoticon = Emoticons.ops()
        output = ""
        #output  = Style.ICYAN + "=".ljust(len(title)+5,"=")
        output = f"\n{Style.IBLUE} {emoticon} {Style.IGREEN}{title}{Style.RESET}\n"
        #output += Style.ICYAN + "=".ljust(len(title)+5,"=")
        if msg:
           output += f"{Style.GREEN}{msg}"
        return output
    def showWarning(title,msg=None):
        output = Messages.formatWarning(title,msg)
        output = Utils.removeCharsColors(output) if Preferences().noColor else output
        printPy(output)

    def formatStartExecution(title,msg=None):
        emoticon = Emoticons.pointRight()
        output = ""
        output += f"{Style.IBLUE} {emoticon} {Style.IGREEN}{title}{Style.RESET}"
        if msg:
           output += f"{Style.GREEN}{msg}"
        return output
    def showStartExecution(title,msg=None):
        output = Messages.formatStartExecution(title,msg)
        output = Utils.removeCharsColors(output) if Preferences().noColor else output
        printPy(output, end='\r')
    
    @staticmethod
    def formatError(title,msg=None):
        emoticon = Emoticons.ops()
        output = ""
        output = f"\n{Style.IRED} {emoticon} {Style.IMAGENTA}{title}{Style.RESET}\n"
        if msg:
           output += f"    {Style.GREEN}{msg}"
        return output
    def showError(title,msg=None):
        output = Messages.formatError(title,msg)
        output = Utils.removeCharsColors(output) if Preferences().noColor else output
        printPy(output)
        printPy("")




from awsee.style import Style
from awsee.emoticons import Emoticons
from awsee.awseestate import AwsSeeState
from awsee.general import *

class Functions:

    LIST_PROFILES             = "LIST_PROFILES"
    SYNC                      = "SYNC"
    INFO                      = "INFO"
    CLEAN                     = "CLEAN"
    ENV                       = "ENV"

    ADD_ROLE                  = "ADD_ROLE"
    REMOVE_ROLE               = "REMOVE_ROLE"
    LIST_ROLES                = "LIST_ROLES"

    ADD_MFA                   = "ADD_MFA"
    REMOVE_MFA                = "REMOVE_MFA"
    LIST_MFAS                 = "LIST_MFAS"
    START_SESSION_MFA_TOKEN   = "START_SESSION_MFA_TOKEN"
    START_SESSION             = "START_SESSION"

    ASSUME_ROLE               = "ASSUME_ROLE"

    # Groups
    GENERAL     = "GENERAL"
    ROLE        = "ROLE"
    MFA         = "MFA"
    AWS_SESSION = "AWS SESSION"

    FUNCTIONS = {
        INFO:{
            "name": "Info about your session",
            "identifiers": ["-i","--info"],
            "group": GENERAL,
            "description": "Show information about a session opened with AWSee",
            "arguments": [
                #{"name": "full","mandatory": False, "description": "List all the information, the roles added for each profile.","biggerLabel": 20},
            ],
            "examples-interactive": [
                {"command":f"{Style.GREEN}awsee --info {Style.RESET}"},
            ]
        },
        ENV:{
            "name": "Info about AWS Environment Variable",
            "identifiers": ["-e","--env"],
            "group": GENERAL,
            "description": "Show information about all the AWS Environment Variables set",
            "arguments": [
                #{"name": "full","mandatory": False, "description": "List all the information, the roles added for each profile.","biggerLabel": 20},
            ],
            "examples-interactive": [
                {"command":f"{Style.GREEN}awsee --env {Style.RESET}"},
            ]
        },
        CLEAN:{
            "name": "Clean the currennt session",
            "identifiers": ["-c","--clean"],
            "group": GENERAL,
            "description": "Clean the current AWS Session from the environment",
            "arguments": [
                #{"name": "full","mandatory": False, "description": "List all the information, the roles added for each profile.","biggerLabel": 20},
            ],
            "examples-interactive": [
                {"command":f"{Style.GREEN}awsee --clean {Style.RESET}"},
            ]
        },
        LIST_PROFILES:{
            "name": "List profiles",
            "identifiers": ["-l","-lf","--list"],
            "group": GENERAL,
            "description": "List all profiles at your credentials (~/.aws/credentials)",
            "arguments": [
                {"name": "full","mandatory": False, "description": "List all the information, the roles added for each profile.","biggerLabel": 20},
            ],
            "examples-interactive": [
                {"command":f"{Style.GREEN}awsee --list {Style.RESET}"},
                {"command":f"{Style.GREEN}awsee --list {Style.IBLUE}full {Style.RESET}"},
                {"command":f"{Style.GREEN}awsee -l {Style.IBLUE}full {Style.RESET}"},
                {"command":f"{Style.GREEN}awsee -lf {Style.MAGENTA}# Same as -l full / --list full{Style.RESET}"},
            ]
        },
        SYNC:{
            "name": "Synchronize cache with remote data of credentials",
            "identifiers": ["-s","--sync"],
            "group": GENERAL,
            "description": f"Synchronize information remotely acquired from AWS (ex: Account Number, MFA Serial) for the credentials. This is done automatically (if not blocked by Polices) only once, when the {Style.IGREEN}awsee{Style.RESET} run for the first time. Using this command, you can force an update.",
            "arguments": [
                # {"name": "-f","mandatory": False, "description": "Full list, MFA Serial and Account (remote calls)","biggerLabel": 20}
            ],
            "examples-interactive": [
                {"command":f"{Style.GREEN}awsee --sync {Style.RESET}"},
            ]
        },
        START_SESSION_MFA_TOKEN:{
            "id": START_SESSION_MFA_TOKEN,
            "identifiers": [],
            "name": "Start AWS Session using MFA Token",
            "group": AWS_SESSION,
            "description": f"Start an AWS Session that needs your six-digit MFA Token. Optionally, it can be informed the profile chosen, if not informe the default will be used.",
            "arguments": [
                {"name": "{000000}","mandatory": True, "description": "The MFA Token","biggerLabel": 20},
                {"name": "[profile]","mandatory": False, "description": "The profile to be used to start the Session","biggerLabel": 20}
            ],
            "examples-interactive": [
                {"command":f"{Style.GREEN}awsee 874341 {Style.RESET}"},
                {"command":f"{Style.GREEN}awsee 687509 {Style.RESET}{Style.IBLUE}johneast{Style.RESET}"},
            ]
        },
        START_SESSION:{
            "id": START_SESSION,
            "identifiers": ["-p"],
            "name": "Start AWS Session",
            "group": AWS_SESSION,
            "description": f"Start an AWS Session for a specific profile.",
            "arguments": [
                {"name": "[profile]","mandatory": True, "description": "The profile to be used to start the Session","biggerLabel": 20}
            ],
            "examples-interactive": [
                {"command":f"{Style.GREEN}awsee -p {Style.RESET}{Style.IBLUE}johneast{Style.RESET}"},
                {"command":f"{Style.GREEN}awsee -p {Style.RESET}{Style.IBLUE}supplier_cc{Style.RESET}"},
            ]
        },
         ASSUME_ROLE:{
            "name": "Assume Role",
            "group": AWS_SESSION,
            "identifiers": ["-r","--assume-role"],
            "description": f"Assume a role",
            "arguments": [
                 {"name": "{role}","mandatory": True, "description": "The role name to be assumed (check their names listing the roles using: aswme -lr","biggerLabel": 20},
            ],
            "examples-interactive": [
                {"command":f"{Style.GREEN}awsee -r {Style.RESET}{Style.IBLUE}supplier{Style.RESET}"},
                {"command":f"{Style.GREEN}awsee -r {Style.RESET}{Style.IBLUE}ecommerce{Style.RESET}"},
            ]
        },
        ADD_ROLE:{
            "name": "Add a role to one of your profiles",
            "identifiers": ["-ar","--add-role"],
            "group": ROLE,
            "description": f"Associate a Role to one of your profiles to enable the request of Session Tokens while \"assuming\" this role (ex: cross account access).",
            "arguments": [
                # {"name": "[profile]","mandatory": False, "description": "The profile to be used to start the Session","biggerLabel": 20}
            ],
            "examples-interactive": [
                {"command":f"{Style.GREEN}awsee -ar {Style.RESET}"},
            ]
        },
        LIST_ROLES:{
            "name": "List roles of your profiles",
            "identifiers": ["-lr","--list-role"],
            "group": ROLE,
            "description": f"List all roles added to use with your profiles.",
            "arguments": [
                 {"name": "[profile]","mandatory": False, "description": "Filter by profile","biggerLabel": 20}
            ],
            "examples-interactive": [
                {"command":f"{Style.GREEN}awsee -lr {Style.RESET}"},
                {"command":f"{Style.GREEN}awsee -lr {Style.RESET}{Style.IBLUE}johneast{Style.RESET}"},
            ]
        },
        REMOVE_ROLE:{
            "name": "Remove a role of your profile",
            "identifiers": ["-rr","--remove-role"],
            "group": ROLE,
            "description": f"Remove a role added to your profile.",
            "arguments": [
                 {"name": "[#number]","mandatory": False, "description": "Number of the role in the list to be removed. Use -lr to see the list","biggerLabel": 20},
                 {"name": "[role-name]","mandatory": False, "description": "Name of the role to be removed","biggerLabel": 20}
            ],
            "examples-interactive": [
                {"command":f"{Style.GREEN}awsee -rr {Style.RESET}{Style.IBLUE}2{Style.RESET}"},
                {"command":f"{Style.GREEN}awsee -rr {Style.RESET}{Style.IBLUE}johnwest{Style.RESET}"},
            ]
        },
        ADD_MFA:{
            "name": "Add a MFA Device to one of your profiles",
            "identifiers": ["-am","--add-mfa"],
            "group": MFA,
            "description": f"Associate a MFA Device to one of your profiles to enable access using MFA Tokens.",
            "arguments": [
                # {"name": "[profile]","mandatory": False, "description": "The profile to be used to start the Session","biggerLabel": 20}
            ],
            "examples-interactive": [
                {"command":f"{Style.GREEN}awsee -am {Style.RESET}"},
            ]
        },
        LIST_MFAS:{
            "name": "List MFAs Device of your profiles",
            "identifiers": ["-lm","--list-mfa"],
            "group": MFA,
            "description": f"List all MFA Devices added to use with your profiles.",
            "arguments": [
                 {"name": "[profile]","mandatory": False, "description": "Filter by profile","biggerLabel": 20}
            ],
            "examples-interactive": [
                {"command":f"{Style.GREEN}awsee -lm {Style.RESET}"},
                {"command":f"{Style.GREEN}awsee -lm {Style.RESET}{Style.IBLUE}johneast{Style.RESET}"},
            ]
        },
        REMOVE_MFA:{
            "name": "Remove a MFA Device of your profile",
            "group": MFA,
            "identifiers": ["-rm","--remove-mfa"],
            "description": f"Remove a MFA Device added to your profile.",
            "arguments": [
                 {"name": "[#number]","mandatory": True, "description": "Number of the MFA Device in the list to be removed. Use -lm to see the list","biggerLabel": 20},
            ],
            "examples-interactive": [
                {"command":f"{Style.GREEN}awsee -rm {Style.RESET}{Style.IBLUE}3{Style.RESET}"},
            ]
        }
    }

    def __init__(self):
        pass

    def showUsage(self, short=True):
        SIZE_SEPARATOR             = 120
        LABEL_LENGTH_FUNCTION_NAME = 25

        output  = "-".ljust(SIZE_SEPARATOR,"-") + "\n" 
        output += "   " + Emoticons.tool() + "   " + Style.CYAN + "U S A G E" + Style.RESET + "" + "\n"
        output += "-".ljust(SIZE_SEPARATOR,"-") + "\n"
        output += f"""
        {Style.GREEN}awsee {Style.IBLUE}[TOKEN-MFA]{Style.RESET} {Style.IBLUE}[-p profile] [-r role] {Style.IBLACK}[-nt or --new-terminal]
        
        Examples:
            {Style.GREEN}awsee {Style.IBLUE}857946                       {Style.IMAGENTA}Open AWS Session with default profile with your six-digit MFA Token
            {Style.GREEN}awsee {Style.IBLUE}857946 -p infra              {Style.IMAGENTA}Open AWS Session with the passed profile using your six-digit MFA Token
            {Style.GREEN}awsee {Style.IBLUE}857946 -p infra -r prod      {Style.IMAGENTA}Open AWS Session with the passed profile using your six-digit MFA Token
                                               and immediatelly assume the passed role at this new session
            {Style.GREEN}awsee{Style.IBLUE}                              {Style.IMAGENTA}Open AWS Session with default profile, it will check if needs MFA 
            {Style.GREEN}awsee {Style.IBLUE}-p dev                       {Style.IMAGENTA}Open AWS Session with the passed profile, asks for MFA if needed
            {Style.GREEN}awsee {Style.IBLUE}-r developer                 {Style.IMAGENTA}Assume a Role inside the current AWS Session
            {Style.GREEN}awsee {Style.IBLUE}-p dev  -r developer         {Style.IMAGENTA}Open AWS Session with the passed profile, and following assume the role 
            {Style.GREEN}awsee {Style.IBLUE}-p prod -r admin             {Style.IMAGENTA}Open AWS Session with the passed profile, and following assume the role 
        
        {Style.GREEN}awsee -nt {Style.RESET}or {Style.GREEN}--new-terminal            {Style.IMAGENTA}When opening an AWS Session, will do it in a new terminal{Style.RESET}
        {Style.GREEN}awsee -h  {Style.RESET}or {Style.GREEN}--help                    {Style.IMAGENTA}Show this and all auxiliary functions (use --help for more information){Style.RESET}
        {Style.GREEN}awsee -u  {Style.RESET}or {Style.GREEN}--usage                   {Style.IMAGENTA}Show only how to use, not the auxiliary functions{Style.RESET}

        {Style.GREEN}awsee {Style.IBLUE}[FUNCTION] {Style.IMAGENTA}                      {Style.IMAGENTA}Execute an auxiliary function, see below for all available options{Style.RESET}
        """
        printPy(output)

    def showFunctions(self, short=True):
        SIZE_SEPARATOR             = 120
        LABEL_LENGTH_FUNCTION_NAME = 25

        output  = "-".ljust(SIZE_SEPARATOR,"-") + "\n" 
        output += "   " + Emoticons.tool() + "   " + Style.CYAN + "F U N C T I O N S" + Style.RESET + "" + "\n"
        output += "-".ljust(SIZE_SEPARATOR,"-") + "\n"
        groupPrinted = ""
        i = 0
        for key in self.FUNCTIONS:
            i += 1
            function      = self.FUNCTIONS[key]
            functionName  = self.convertFunctionsName(key)
            functionDescr = "  " + Emoticons.pointRight() + " " + Style.GREEN + functionName + Style.RESET + (" ".ljust(LABEL_LENGTH_FUNCTION_NAME - len(functionName) + 2,"-")) + \
                                  "> "+ Functions.formatDescr(function["description"], 34, 60) + "\n"
            
            groupTitle = " " + Style.IWHITE + Style.BG_BLUE + " --> " + function["group"].upper() + " ".ljust(10 - len(function["group"])," ") + Style.RESET + "\n" #+ "-".ljust(len(function["group"]),"-") + "\n"    
            if groupPrinted != function["group"]:
                output += groupTitle 
                groupPrinted = function["group"]

            output += functionDescr

            if not short:
               if len(function["arguments"]) > 0:
                  output += Style.IBLUE + "          Arguments: " + Style.RESET + "\n"
               for args in function["arguments"]:
                   output += Style.IBLUE + "               " + args["name"] + " ".ljust(args["biggerLabel"] - len(args["name"])," ") + Style.IBLUE 
                   output += ("[Required]" if args["mandatory"] == True else "[Optional]")
                   output += Style.RESET + "....: " + Style.RESET + Functions.formatDescr(args["description"],51,45) + "\n"    
               output += "          " + Style.IBLUE + "Examples:" + Style.RESET + "\n"
               for exs in function["examples-interactive"]:
                   output += "               " + exs["command"] + "\n"
               output += "-".ljust(SIZE_SEPARATOR,"-") + "\n"
            else:
               output += " ".ljust(LABEL_LENGTH_FUNCTION_NAME + 10," ") + f"{Style.GREEN}awsee "
               for ii,ident in enumerate(function["identifiers"]):
                   output += f"{Style.GREEN}[{ident}]"
                   output += " or " if ii < len(function["identifiers"]) - 1 else ""
               output += f"{Style.RESET}\n"
            
        printPy(output)
    
    def showFunction(self, key):
        SIZE_SEPARATOR             = 95
        LABEL_LENGTH_FUNCTION_NAME = 25

        function = Functions.FUNCTIONS[key]

        output  = "-".ljust(SIZE_SEPARATOR,"-") + "\n"           
        output += "   " + Emoticons.tool() + " " + Style.CYAN +  function["name"].upper() + Style.RESET + "" + "\n"
        output += "-".ljust(SIZE_SEPARATOR,"-") + "\n"    

        functionName = self.convertFunctionsName(key)
        functionDescr = "  " + Emoticons.pointRight() + " " + Style.GREEN + functionName + Style.RESET + (" ".ljust(LABEL_LENGTH_FUNCTION_NAME - len(functionName) + 2,"-")) + \
                                "> "+ Functions.formatDescr(function["description"], 26, 59) + "\n"
        output += functionDescr
        output += Style.MAGENTA + "          Arguments: " + Style.RESET + "\n"
        for args in function["arguments"]:
            output += Style.BLUE + "               " + args["name"] + " ".ljust(args["biggerLabel"] - len(args["name"])," ") + Style.MAGENTA 
            output += ("[Required]" if args["mandatory"] == True else "[Optional]")
            output += Style.RESET + "....: " + Style.RESET + Functions.formatDescr(args["description"],51,34) + "\n"
        output += "          " + Style.MAGENTA + "Examples:" + Style.RESET + "\n"
        for exs in function["examples-interactive"]:
            output += "               " + exs["command"] + "\n"
        output += "-".ljust(SIZE_SEPARATOR,"-") + "\n"
        return output

    def getFunctionByArgumentIdentifier(self,identifier):
        for f in self.FUNCTIONS:
            for identf in self.FUNCTIONS[f]["identifiers"]:
                if identf == identifier:
                   return f
        return None

    def convertFunctionsName(self,function):
        pos = function.find("-")
        while pos > 1 and pos < 20:
            capitalize = function[pos+1]
            function = function.replace("-"+capitalize,capitalize.upper())
            pos = function.find("-")
        return function

    def formatDescr(text, margin, width):
        MARGIN    = margin
        WIDTH     = width
        formatted = text
        line      = ""

        if len(formatted) > WIDTH:
           formatted = ""
           for word in text.split(" "):
               if len(line + word + " ") <= (WIDTH):
                  line += word + " "
               else:
                  finalLine  = Functions.fillLineSpacesUntil(line,WIDTH)
                  formatted += finalLine + "\n" + "".ljust(MARGIN," ") 
                  line = word + " " 
        
        return formatted + line
    
    def fillLineSpacesUntil(line, width):
        finalLine = line
        posSpace  = 0
        while (len(finalLine) + 1)  < width:
            spaceIndex = finalLine.find(' ',posSpace)
            finalLine  = finalLine[:spaceIndex] + " " + finalLine[spaceIndex:]
            posSpace   = spaceIndex + 2
           
        return finalLine      




import pathlib
from setuptools import setup,find_packages

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# 
setup(
    name="awsee",
    version="0.0.15",
    description="Utility to help me manage my AWS profiles",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://ualter.github.io/awsee-site/",
    author="Ualter Otoni Pereira",
    author_email="ualter.junior@gmail.com",
    keywords = ['aws', 'cloud', 'awsee', 'boto3', 'ec2', 'vpc'],
    license="MIT",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.7",
    ],
    packages=["awsee"],
    include_package_data=True,
    install_requires=[            
        'botocore>=1.17.63','boto3>=1.14.2','Pygments','arnparse','tinydb','clipboard','pyreadline','pytz','python-dateutil','XlsxWriter'
    ],
    scripts=[
        'shell_scripts/awsee.bat',
        'shell_scripts/awsee'
    ],
    entry_points={
        "console_scripts": [
            "awseepy=awsee.awseepy:main",
        ]
    },
)


import os
import configparser
from os.path import expanduser
from awsee.emoticons import Emoticons

USER_DIR    = os.path.join(expanduser("~"),".awsee")
FILE_INI    = USER_DIR + "awsee.ini"
GENERAL     = "GENERAL"
WINDOWS     = "WINDOWS"
LINUX       = "LINUX"

class Preferences:

    @property
    def defaultProfile(self):
        return self._defaultProfile
    
    @property
    def emoticonsEnabled(self):
        return self._emoticonsEnabled
    
    @property
    def noColor(self):
        return self._noColor

    @property
    def windows(self):
        return self._windows

    @property
    def linux(self):
        return self._linux

    def __init__(self):
        if not os.path.exists(FILE_INI):
            configFileIni = configparser.ConfigParser(allow_no_value=True)
            configFileIni.add_section(GENERAL)
            configFileIni.add_section(WINDOWS)
            configFileIni.add_section(LINUX)
            configFileIni.set(GENERAL, "default-profile","default")
            configFileIni.set(GENERAL, "emoticons-enabled","true")
            configFileIni.set(GENERAL, "no-color","false")

            configFileIni.set(WINDOWS, "; WINDOWS Options for New Terminal Window")
            configFileIni.set(WINDOWS, ";   Leave it blanks to use the default options CMDER or CMD")
            configFileIni.set(WINDOWS, ";   To use Git-Bash, use option 1 or 2 pointing to your installation path")
            configFileIni.set(WINDOWS, ";   To use Windows Terminal, use option 3")
            configFileIni.set(WINDOWS, "; Option 0) terminal-command =")
            configFileIni.set(WINDOWS, "; Option 1) terminal-command = C:\\Program Files\\Git\\apps\\Git\\bin\\sh.exe")
            configFileIni.set(WINDOWS, "; Option 2) terminal-command = C:\\Program Files\\Git\\apps\\Git\\git-bash.exe")
            configFileIni.set(WINDOWS, "; Option 3) terminal-command = wt")
            configFileIni.set(WINDOWS, "terminal-command","")

            configFileIni.set(LINUX, "; LINUX Options for New Terminal Window")
            configFileIni.set(LINUX, ";   Leave it blanks to use the default option")
            configFileIni.set(LINUX, ";   ---")
            configFileIni.set(LINUX, "; Option 0) terminal-command =")
            configFileIni.set(LINUX, "; Option 1) terminal-command = ")
            configFileIni.set(LINUX, "terminal-command","")

            with open(FILE_INI,'w') as configfile:
                configFileIni.write(configfile)
                configFileIni = configparser.ConfigParser(allow_no_value=True)
        
        configFileIni = configparser.ConfigParser()
        configFileIni.read(FILE_INI)
        self._defaultProfile   = configFileIni[GENERAL]["default-profile"]
        self._emoticonsEnabled = configFileIni[GENERAL]["emoticons-enabled"] in ['True','true']
        self._noColor          = configFileIni[GENERAL]["no-color"] in ['True','true'] 
        self._windows          = Windows(configFileIni[WINDOWS]["terminal-command"])
        self._linux            = Windows(configFileIni[LINUX]["terminal-command"])

        Emoticons.ENABLED = self._emoticonsEnabled

class Windows:

    @property
    def terminalCommand(self):
        return self._terminal_command

    def __init__(self, _terminal_command):
        self._terminal_command = _terminal_command

class Linux:

    @property
    def terminalCommand(self):
        return self._terminal_command

    def __init__(self, _terminal_command):
        self._terminal_command = _terminal_command



import os, uuid
import datetime
from boto3 import session
from botocore import credentials
from awsee.awseestate import AwsSeeState
from arnparse import arnparse
from os.path import expanduser
from awsee.logmanager import LogManager
from awsee.preferences import Preferences
from awsee.functions import Functions
from awsee.awsservices import AwsServices
from awsee.messages import Messages
from awsee.tableargs import TableArgs
from awsee.prettytable import PrettyTable
from awsee.style import Style
from awsee.emoticons import Emoticons
from awsee.utils import Utils
from tinydb import Query, where
from shutil import which
from awsee.repository import Repository, CredentialsRepository, ConfigRepository, RoleRepository, MfaRepository
from awsee.general import *
from awsee.sessioninfo import SessionInfo, CallerIdentity

class SessionManager:

    SCRIPTS_PREFIX_NAME_FILE = "set-awssession-"

    def __init__(self):
        self.credentialsRepository = CredentialsRepository()
        self.roleRepository        = RoleRepository()
        self.mfaRepository         = MfaRepository()
        self.preferences           = Preferences()

    # Assume a Role
    #def assumeRole(self, role, profile=None, mfaToken=None):
    def assumeRole(self, sessionInfo: SessionInfo):
        aws = AwsServices()
        
        stsToken, callerIdentity = aws.assumeRole(sessionInfo.roleArn, sessionInfo.profile, sessionInfo.mfaToken)
        if not stsToken:
            return None, None
        sessionInfo.stsToken       = stsToken["Credentials"]
        sessionInfo.callerIdentity = callerIdentity
        sessionInfo.profile        = sessionInfo.roleProfile
        
        return self.createAWSSessionOnTerminalUser(sessionInfo)
    
    # Open Session using MFA (default or other profile)
    def startAWSSession(self, sessionInfo: SessionInfo):
        aws = AwsServices()

        _stsToken                                   = {}
        _stsToken["Credentials"]                    = {}
        _stsToken["Credentials"]["AccessKeyId"]     = sessionInfo.credentials['accessKey']
        _stsToken["Credentials"]["SecretAccessKey"] = sessionInfo.credentials['secretKey']
        _stsToken["Credentials"]["SessionToken"]    = ""
        _stsToken["Credentials"]["Expiration"]      = ""

        callerIdentity = aws.getCallerIdentity(_stsToken)

        sessionInfo.stsToken       = _stsToken["Credentials"]
        sessionInfo.callerIdentity = callerIdentity

        return self.createAWSSessionOnTerminalUser(sessionInfo)

    # Open Session (other profile - no need MFA)
    #def startAWSSessionWithMFA(self, profile, mfa, mfaToken, role=None):
    def startAWSSessionWithMFA(self, sessionInfo: SessionInfo):
        aws = AwsServices()
        
        stsTokenSession, callerIdentitySession = aws.getSessionToken(mfaSerial=sessionInfo.mfaDevice, mfaToken=sessionInfo.mfaToken, profile=sessionInfo.profile)
        if not stsTokenSession:
            return None, None

        if sessionInfo.hasRole:
           # If a Role was also passed, 
           # Initiate a session the the Profile given, and using it, immediately, create a Session with the Role passed
           stsTokenSessionRole, callerIdentitySessionRole = aws.startSessionWithTokenAndAssumeRole(stsTokenSession, sessionInfo.roleArn)
           sessionInfo.stsToken                           = stsTokenSessionRole["Credentials"]
           sessionInfo.callerIdentity                     = callerIdentitySessionRole
        else:
           sessionInfo.stsToken       = stsTokenSession["Credentials"]
           sessionInfo.callerIdentity = callerIdentitySession
        
        return self.createAWSSessionOnTerminalUser(sessionInfo)

    def createAWSSessionOnTerminalUser(self, sessionInfo: SessionInfo):
        output                     = None
        fileNamerole               = "" if not sessionInfo.hasRole else "_" + sessionInfo.roleName.replace(" ","_").lower()
        fileNameProfile            = "" if not sessionInfo.profile else sessionInfo.profile.replace(" ","_")
        roleTitle                  = "" if not sessionInfo.hasRole else f"Role: {sessionInfo.roleName.replace('_','')}" 
        windowTitle                = f"AWSEE Profile: {fileNameProfile}  {roleTitle}"
        sessionInfo.scriptFileWin  = f"{self.SCRIPTS_PREFIX_NAME_FILE}{fileNameProfile}{fileNamerole}.bat"
        sessionInfo.scriptFileBash = f"{self.SCRIPTS_PREFIX_NAME_FILE}{fileNameProfile}{fileNamerole}.sh"
        

        # Check if Config was loaded, otherwise load it if exists
        if not sessionInfo.hasConfig:
           sessionInfo.config = ConfigRepository().findByProfile(sessionInfo.profile)

        if Utils.isWindows():
            terminalCommand            = self.preferences.windows.terminalCommand
            sessionInfo.setBashEnv     = False
            outputWin, environmentsWin = self.createEnvironment(sessionInfo)
            output = outputWin
            
            # Check if there's a Terminal Commmand Set
            if AwsSeeState.NEW_TERMINAL:
                if not terminalCommand or terminalCommand.lstrip().rstrip() == "":
                    # Check CMDER is installed (then use it)
                    if which("cmder") and os.environ['CMDER_ROOT'] != "":
                        os.system(f"start %CMDER_ROOT%\\vendor\\conemu-maximus5\\ConEmu.exe /icon \"%CMDER_ROOT%\\cmder.exe\" /title \"{windowTitle}\" /loadcfgfile \"%CMDER_ROOT%\\config\\ConEmu.xml\" /cmd cmd /k " + environmentsWin)
                    else:
                        # Last option (windows default cmd)
                        if which("cmd"):
                            os.system("start cmd.exe /c cmd /k" + environmentsWin)
                        else:
                            msg = f"""\n{Style.GREEN}    Please set a {Style.IBLUE}terminal-command{Style.GREEN} at your {Style.IBLUE}~/.awsee/awsee.ini{Style.GREEN}, WINDOWS section in order \n    to open a Terminal ready to use with your AWS Session Token configured.
                            \n    Example: \n    {Style.IBLUE}[WINDOWS]\n    terminal-command = {Style.IGREEN}d:\Program Files\Git\git-bash.exe{Style.RESET}
                            """
                            Messages.showError("No Terminal Found (CMDER or CMD)",msg)
                            return None, None
                else:
                    if "git-bash.exe" in terminalCommand or "sh.exe" in terminalCommand:
                        sessionInfo.setBashEnv = True
                        outputBash, environmentsBash = self.createEnvironment(sessionInfo)
                        output = outputBash

                        # Regarding the place where Python is running (not the Target Console to Open, configured at awsee.ini)...  
                        # Check if the Call to open a Shell Script Terminal were made inside a GitBash, if not... change to Windows Style (the Output Python window)
                        if not Utils.isRunningOnGitBash():
                            outputWin, _ = self.createEnvironment(sessionInfo)
                            output = outputWin

                        os_command = ""
                        if "git-bash.exe" in terminalCommand:
                            os_command = f"start {terminalCommand} --needs-console --no-hide --command=usr\\bin\\bash.exe --login -i -c \"sh -c 'source ./{environmentsBash}; exec sh'\""
                        elif "sh.exe" in terminalCommand:
                            os_command = f"start {terminalCommand} --login -i -l -c \"sh -c 'source ./{environmentsBash}; exec sh'\""
                        else:
                            msg = f"""\n{Style.GREEN}    Please check the {Style.IBLUE}terminal-command{Style.GREEN} set at your {Style.IBLUE}~/.awsee/awsee.ini{Style.GREEN}, WINDOWS section in order \n    to open a Terminal ready to use with your AWS Session Token configured.
                            \n    Example: \n    {Style.IBLUE}[WINDOWS]\n    terminal-command = {Style.IGREEN}d:\Program Files\Git\sh.exe{Style.RESET}"""
                            Messages.showError(f"Terminal Command set \"{terminalCommand}\" not supported (Use git-bash.exe, sh.exe or let it blank, for system default",msg)
                            return None, None
                        os.system(os_command)
                    elif "wt" in terminalCommand:
                        os.system("wt -d c:\ cmd /k " + environmentsWin)

        else:
            # LINUX
            terminalCommand                = self.preferences.linux.terminalCommand
            sessionInfo.setBashEnv         = True
            outputLinux, environmentsLinux = self.createEnvironment(sessionInfo)
            output                         = outputLinux

            if AwsSeeState.NEW_TERMINAL:
                Messages.showWarning("New Terminal option not implemented here, yet!")
                sys.exit()
            #    if "cmd" in terminalCommand:
            #        # Using WSL (User inside Windows)
            #        sessionInfo.setBashEnv = False
            #        _, environmentsWin = self.createEnvironment(sessionInfo)
            #        printPy(environmentsWin)
            #        os.system("cmd.exe /c start cmd /k " + environmentsWin)
               
                

        return output, sessionInfo

    def cleanForEnvironmentVariables(self, isWindows) -> str:
        output = ""
        if isWindows:
           output += "set AWS_ACCESS_KEY_ID=\n"
           output += "set AWS_SECRET_ACCESS_KEY=\n"
           output += "set AWS_SESSION_TOKEN=\n"
           output += "set AWS_EXPIRATION=\n"
           output += "set AWS_SEE=CLOSED\n"
           output += "set AWS_WITH_TOKEN=\n"
           output += "set AWS_ROLE=\n"
           output += "set AWS_USER_ARN=\n"
           output += "set AWS_USER_ID=\n"
        else:
           output += "unset AWS_ACCESS_KEY_ID\n"
           output += "unset AWS_SECRET_ACCESS_KEY\n"
           output += "unset AWS_SESSION_TOKEN\n"
           output += "unset AWS_EXPIRATION\n"
           output += "unset AWS_ROLE\n"
           output += "unset AWS_WITH_TOKEN\n"
           output += "export AWS_SEE=CLOSED\n"
           output += "unset AWS_USER_ARN\n"
           output += "unset AWS_USER_ID\n"
        return output

    # Script to Help clean environment variables, etc. 
    def createCleanEnvironmentScript(self, scriptFile):
        if ".bat" in scriptFile:
            enviromentVariables = self.cleanForEnvironmentVariables(False).replace("\n","")
            cleanFileName = "clean.bat"
            content = f"""@ECHO OFF
{enviromentVariables}
DEL {self.SCRIPTS_PREFIX_NAME_FILE}*
            """
        else:
            enviromentVariables = self.cleanForEnvironmentVariables(True).replace("\n","")
            cleanFileName = "clean.sh"
            content = f"""{enviromentVariables}
rm {self.SCRIPTS_PREFIX_NAME_FILE}*
            """
        completePathFileName =  os.path.join(TEMP_CACHE_FOLDER,cleanFileName)
        with open(f"{completePathFileName}",'w') as fout:
            fout.write(content)

    # Create Contents, Script File for Environments Variables and Clipboard (Win and Bash)
    def createEnvironment(self, sessionInfo: SessionInfo):
        output           = self.createOutputStartSession(sessionInfo)
        self.addEnvironmentVariablesConfigurationToClipboard(sessionInfo)
        environments     = ""
        if AwsSeeState.NEW_TERMINAL:
            initFileContent  = self.createContentScriptFileStartSession(sessionInfo)
            completePathFile = os.path.join(TEMP_CACHE_FOLDER, sessionInfo.scriptFile)
            with open(f"{completePathFile}",'w') as fout:
                fout.write(initFileContent)
            environments = f"\"{completePathFile}\""
            self.createCleanEnvironmentScript(completePathFile)
        return output, environments

    def calculateExpiresTime(self, expiration):
        if expiration and expiration != "":
            diff      = (expiration - datetime.datetime.now().astimezone())
            if diff.seconds > 60:
                min  = int(diff.seconds / 60)
                secs = int(((diff.seconds / 60) - min) * 60)
                return "00:{:02d}:{:02d} minutes".format(min, secs)
            else:   
                return "00:00:{:02d} seconds".format(diff.seconds)
        return ""

    #
    # This is the information to the user about his/her AWS Session opened, just informative
    #
    def createOutputStartSession(self, sessionInfo: SessionInfo):
        commandEnvSet = "set"
        executeScript = f"{sessionInfo.scriptFileWin}"
        if sessionInfo.isBashEnv:
            commandEnvSet = "export"
            executeScript = f"{sessionInfo.scriptFileBash}"
            

        diffLabel = self.calculateExpiresTime(sessionInfo.expiration)

        credential = self.credentialsRepository.findByProfile(sessionInfo.profile)
        awsProfile = f"{Style.GREEN}Profile................: {Style.IBLUE}{sessionInfo.profile}" if sessionInfo.hasProfile else ""
        awsRole    = f"{Style.GREEN}Role...................: {Style.IBLUE}{sessionInfo.roleName}" if sessionInfo.hasRole else ""
        awsAccount = f"{Style.GREEN}Account................: {Style.IBLUE}{credential['account']}"

        output  = "\n"
        output += f" {Style.IMAGENTA}#aswee\n"
        output += f" {Style.GREEN}======================="
        output += f"\n {Style.IMAGENTA}  >>> AWS SESSION <<<\n"
        output += f" {Style.GREEN}======================="
        if diffLabel:
           output += f"\n {Style.GREEN}Session will expires...: {Style.IBLUE}{diffLabel}"
        output += f"\n{Style.GREEN} {awsAccount}"
        output += f"\n{Style.GREEN} {awsProfile}"
        output += f"\n{Style.GREEN} {awsRole}"

        if AwsSeeState().NEW_TERMINAL:
            output += f"\n\n {Style.IBLUE}--> {Style.IMAGENTA}Session opened in another terminal..."

        output += f"{Style.RESET}"
        return output

    # This is the same AWS Session credentials information for the user in case he/she wants to open using the Clipboard
    def addEnvironmentVariablesConfigurationToClipboard(self, sessionInfo: SessionInfo):
        command = "set" if not sessionInfo.isBashEnv else "export"
        awsToken = f"{Style.GREEN}{command} AWS_SESSION_TOKEN{Style.WHITE}={Style.IBLUE}{sessionInfo.sessionToken}\r" if sessionInfo.hasSessionToken else ""
        # Add to Clipboard (SETs environmentsWin)
        forClipboard  = f"{command} AWS_ACCESS_KEY_ID={sessionInfo.accessKey}\r\n"
        forClipboard += f"{command} AWS_SECRET_ACCESS_KEY={sessionInfo.secretKey}\r\n"
        forClipboard += f"{awsToken}\r\n"
        forClipboard += f"{command} AWS_SEE=gG99USedshwRIVSe\r\n"
        if sessionInfo.isBashEnv:
           forClipboard += f"{command} AWS_EXPIRATION=\"{sessionInfo.expiration}\"\r\n"
        else:
           forClipboard += f"{command} AWS_EXPIRATION={sessionInfo.expiration}\r\n" 
        Utils.addToClipboard(forClipboard)

    # This is the content of a Script File (Bath or Bash) to opened a AWS Session with the credentials provided 
    def createContentScriptFileStartSession(self, sessionInfo: SessionInfo):
        initFileContent = self.createBashScriptFile(sessionInfo) if sessionInfo.isBashEnv else \
                          self.createCMDscriptFile(sessionInfo)
        return initFileContent

    def createCMDscriptFile(self, sessionInfo: SessionInfo):
        awsToken = f"set AWS_SESSION_TOKEN={sessionInfo.sessionToken}" if sessionInfo.hasSessionToken else ""
        roleEcho = f"ECHO.Role.........: {sessionInfo.roleName}" if sessionInfo.hasRole else ""
        roleSet  = f"{sessionInfo.roleName}" if sessionInfo.hasRole else " " 

        expirationLabel = self.calculateExpiresTime(sessionInfo.expiration)

        aliasLines  = ""
        for a in self.getAliasFileContent():
            aliasLines += a

        return f"""
        @ECHO OFF
        ECHO.#awsee
        ECHO.=============================
        ECHO.  AWSCLI Session Configured
        ECHO.=============================
        set AWS_ACCESS_KEY_ID={sessionInfo.accessKey}
        set AWS_SECRET_ACCESS_KEY={sessionInfo.secretKey}
        {awsToken}
        set AWS_EXPIRATION={sessionInfo.expiration:{"" if sessionInfo.hasExpiration else ""}}

        for /F "delims=" %%i in ('aws sts get-caller-identity') do (
            SET "string=%%i"
            echo %%i | findstr /C:"Account">nul && (
                SET ACCOUNT=%%i
            )
            echo %%i | findstr /C:"Arn">nul && (
                SET USERARN=%%i
            )
        )
        set ACCOUNT=%ACCOUNT:Account=%
        set ACCOUNT=%ACCOUNT::=%
        set ACCOUNT=%ACCOUNT:"=%
        set ACCOUNT=%ACCOUNT:,=%
        set ACCOUNT=%ACCOUNT: =%

        set USERARN=%USERARN: =%
        set USERARN=%USERARN:~7%
        set USERARN=%USERARN:"=%
        REM set USERARN=%USERARN:Arn=%
        REM set USERARN=%USERARN:,=%

        {aliasLines}
        
        set AWS_ACCOUNT=%ACCOUNT%
        set AWS_USER=%USERARN%
        set AWS_ROLE={roleSet}
        set AWS_SEE=gG99USedshwRIVSe

        ECHO.Expires at...: {expirationLabel}
        ECHO.Account......: %ACCOUNT%
        ECHO.User.........: %USERARN%
        {roleEcho} 
        ECHO.
        """

    def createBashScriptFile(self, sessionInfo: SessionInfo):
        awsToken = f"export AWS_SESSION_TOKEN={sessionInfo.sessionToken}" if sessionInfo.hasSessionToken else ""
        roleEcho = f"echo Role.........: {sessionInfo.roleName}" if sessionInfo.hasRole else " "
        roleSet  = f"{sessionInfo.roleName}" if sessionInfo.hasRole else " " 

        expirationLabel = self.calculateExpiresTime(sessionInfo.expiration)

        aliasLines = ""
        for a in self.getAliasFileContent():
            aliasLines += a
        
        return f"""
        #!/bin/sh
        echo 
        echo =============================
        echo   AWSCLI Session Configured
        echo =============================
        export AWS_ACCESS_KEY_ID={sessionInfo.accessKey}
        export AWS_SECRET_ACCESS_KEY={sessionInfo.secretKey}
        {awsToken}
        export AWS_EXPIRATION="{sessionInfo.expiration:{"" if sessionInfo.hasExpiration else ""}}"

        vlr="$(aws sts get-caller-identity)"
        account=$(echo "$vlr" | grep '"Account"' | sed s/\\"//g | sed s/,//g | sed s/"Account:"//g)
        arn=$(echo "$vlr" | grep '"Arn"' | sed s/\\"//g | sed s/,//g | sed s/"Arn:"//g )

        export AWS_ACCOUNT=$account
        export AWS_USER=$arn
        export AWS_ROLE={roleSet}
        export AWS_SEE=gG99USedshwRIVSe

        {aliasLines}

        echo Expires at...: "{expirationLabel}"
        echo Account......: $account
        echo User.........: $arn
        {roleEcho}
        echo
        """
    
    def createAliasFile(self):
        aliasPath = os.path.join(expanduser("~"),".awsee","alias")
        if not os.path.exists(aliasPath):
            os.makedirs(aliasPath)
        
        aliasFile = os.path.join(aliasPath,"alias_awsee.txt")
        if Utils.isWindows():
            with open(aliasFile,'w') as fout:
                fout.write("DOSKEY info=awsee -i\n")
                fout.write("DOSKEY s3=aws s3 ls\n")
                fout.write("DOSKEY ssmec2=aws ssm start-session --target \"i-\"\n")
                fout.write("DOSKEY ec2=aws ec2 describe-instances --query Reservations[*].Instances[*].InstanceId --output table\n")
        else:
            with open(aliasFile,'w') as fout:
                fout.write("alias s3='aws s3 ls'\n")
                fout.write("alias info='awsee -i'\n")
                fout.write("alias info='aws ssm start-session --target \"i-\"'\n")
                fout.write("alias ec2='aws ec2 describe-instances --query Reservations[*].Instances[*].InstanceId --output table'\n")
    
    def getAliasFileContent(self):
        aliasPath = os.path.join(expanduser("~"),".awsee","alias")
        aliasFile = os.path.join(aliasPath,"alias_awsee.txt")
        with open(aliasFile,'r') as fout:
            lines = list(fout)
        return lines

    def cleanAwsSession(self, showMessage=True):
        isWindows = True if Utils.isWindows() else False
        printOut(self.cleanForEnvironmentVariables(isWindows))
        if showMessage:
           Messages.showMessage("Session cleaned!")

    def setCreateSessionAtLocalOS(self, sessionInfo: SessionInfo, myPidFile: str=None):
        uuidStr = str(uuid.uuid4()).replace("-","")

        os.environ["AWS_SEE"] = uuidStr
        sessionInfo.uuid = uuidStr

        command = "SET" if Utils.isWindows() else "export"

        self.cleanAwsSession(False)

        # Let's clean before set another one (avoid conflicts)
        printOut(f"{command} AWS_ACCESS_KEY_ID={sessionInfo.accessKey}")
        printOut(f"{command} AWS_SECRET_ACCESS_KEY={sessionInfo.secretKey}")
        printOut(f"{command} AWS_SESSION_TOKEN={sessionInfo.sessionToken}")
        if sessionInfo.isBashEnv:
            printOut(f"{command} AWS_EXPIRATION='{sessionInfo.expiration}'")
        else:
            printOut(f"{command} AWS_EXPIRATION={sessionInfo.expiration}")
        printOut(f"{command} AWS_USER_ARN={sessionInfo.callerIdentity.arn}")
        printOut(f"{command} AWS_USER_ID={sessionInfo.callerIdentity.userId}")
        printOut(f"{command} AWS_SEE={uuidStr}")
        printOut(f"{command} AWS_PID={myPidFile if myPidFile else ''}")

        #TODO: Set if config has default region
        if sessionInfo.hasConfig:
            region = sessionInfo.config['configurations']['region']
            printOut(f"{command} AWS_REGION={region}")

        aliasLines = ""
        for a in self.getAliasFileContent():
            aliasLines += a + "\n"
        printOut(f"{aliasLines}")

        if sessionInfo.hasMfaToken:
            printOut(f"{command}  AWS_WITH_TOKEN=yes")
        if sessionInfo.hasProfile:
            printOut(f"{command}  AWS_PROFILE={sessionInfo.profile}")
        if sessionInfo.hasRole:
            printOut(f"{command}  AWS_ROLE={sessionInfo.roleName}")

    def showEnvironment(self):
        prettyTable = PrettyTable(["#","Variable","Value"])
        idx = 0
        for env in os.environ:
            if env.startswith("AWS_"):
                idx += 1
                value = os.environ[env]
                if "SESSION_TOKEN" in env:
                    value = Style.IMAGENTA + "*".ljust(10,"*") + Style.GREEN + value[250:] 
                prettyTable.addRow([idx,env,value])
        tableArgs = TableArgs()
        prettyTable.sortByColumn(int(tableArgs.sortCol) - 1)
        prettyTable.ascendingOrder(not tableArgs.desc)
        output = "\n" + Style.RESET + prettyTable.printMe("environmentVariablesAWsSession",False,tableArgs)
        Messages.showMessage("AWS Session Environment Variables", output)
